from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'

class NSVLANInterfaceBinding(NSBaseResource):

        # General Netscaler configuration object

        def __init__(self, json_data = None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSVLANInterfaceBinding, self).__init__()

                self.options = {
                        'id' : '',
                        'ifnum' : '',
                        'tagged' : '',
                }

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key]=json_data[key]

                self.resourcetype = NSVLANInterfaceBinding.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "vlan_interface_binding"

        def set_id(self, id):
                self.options['id'] = id

        def get_id(self):
                return self.options['id']

        def set_ifnum(self, ifnum):
                self.options['ifnum'] = ifnum

        def get_ifnum(self):
                return self.options['ifnum']

        def set_tagged(self, tagged):
                self.options['tagged'] = tagged

        def get_tagged(self):
                return self.options['tagged']

        @staticmethod
        def add(nitro, resource):
                __resource = NSVLANInterfaceBinding()
                __resource.set_id(resource.get_id())
                __resource.set_ifnum(resource.get_ifnum())
                __resource.set_tagged(resource.get_tagged())
                return __resource.add_resource(nitro)

        @staticmethod
        def delete(nitro, resource):
                __resource = NSVLANInterfaceBinding()
                __resource.set_id(resource.get_id())
                __resource.set_ifnum(resource.get_ifnum())
                __resource.set_tagged(resource.get_tagged())
                nsresponse = __resource.delete_resource(nitro, object_name=__resource.get_id())
                return nsresponse

        @staticmethod
        def get(nitro, resource):
                """
                Use this API to fetch service resource of given name.
                """
                __resource = NSVLANInterfaceBinding()
                __resource.set_id(resource.get_id())
                __resource.get_resource(nitro, object_name=resource.get_id())
                return __resource


from nsbaseresource import NSBaseResource

__author__ = 'Aleksandar Topuzovic'

class NSIPTunnel(NSBaseResource):

        # General Netscaler configuration object

        def __init__(self, json_data = None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSIPTunnel, self).__init__()

                self.options = {
                    "remote": "",
                    "remotesubnetmask": "",
                    "name": "",
                    "sysname": "",
                    "local": "",
                    "protocol": "",
                    "type": "",
                    "encapip": "",
                    "channel": "",
                    "secure": ""
                    }

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key]=json_data[key]

                self.resourcetype = NSIPTunnel.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "iptunnel"

        # Read/write properties
        def set_name(self, name):
            '''
            The name of the ip tunnel.
            Minimum length = 1
            '''
            self.options['name'] = name

        def get_name(self):
            '''
            The name of the ip tunnel.
            Minimum length = 1
            '''
            return self.options['name']

        def set_remote(self, remote):
            '''
            The remote-ip or subnet of the tunnel.
            Minimum length = 1
            '''
            self.options['remote'] = remote

        def get_remote(self):
            '''
            The remote-ip or subnet of the tunnel.
            Minimum length = 1
            '''
            return self.options['remote']


        def set_remotesubnetmask(self, remotesubnetmask):
            '''
            The remote-subnet mask of the tunnel.
            '''
            self.options['remotesubnetmask'] = remotesubnetmask

        def get_remotesubnetmask(self):
            '''
            The remote-subnet mask of the tunnel.
            '''
            return self.options['remotesubnetmask']

        def set_local(self, local):
            '''
            The local-ip of the tunnel.
            '''
            self.options['local'] = local

        def get_local(self):
            '''
            The local-ip of the tunnel.
            '''
            return self.options['local']

        def set_protocol(self, protocol):
            '''
            The IP tunneling protocol.
            Default value: IPIP
            '''
            self.options['protocol'] = protocol

        def get_protocol(self):
            '''
            The IP tunneling protocol.
            Default value: IPIP
            '''
            return self.options['protocol']

        def set_secure(self, secure):
            '''
            Secure communication using IPSec.
            Default value: NO
            '''
            self.options['secure'] = secure

        def get_secure(self):
            '''
            Secure communication using IPSec.
            Default value: NO
            '''
            return self.options['secure']

        # Read only properties
        def get_sysname(self):
            '''
            The name of the ip tunnel.
            '''
            return self.options['sysname']

        def get_type(self):
            '''
            The type of this tunnel.
            '''
            return self.options['type']

        def get_encapip(self):
            '''
            The effective local-ip of the tunnel. Used as the source of the encapsulated packets.
            '''
            return self.options['encapip']

        def get_channel(self):
            '''
            The tunnel which is bound to a netbridge.
            '''
            return self.options['channel']

        @staticmethod
        def add(nitro, iptunnel):
                __iptunnel = NSIPTunnel()
                __iptunnel.set_name(iptunnel.get_name())
                __iptunnel.set_remote(iptunnel.get_remote())
                __iptunnel.set_remotesubnetmask(iptunnel.get_remotesubnetmask())
                __iptunnel.set_local(iptunnel.get_local())
                __iptunnel.set_protocol(iptunnel.get_protocol())
                __iptunnel.set_secure(iptunnel.get_secure())
                return __iptunnel.add_resource(nitro)

        @staticmethod
        def delete(nitro, iptunnel):
                __iptunnel = NSIPTunnel()
                __iptunnel.set_name(iptunnel.get_name())
                nsresponse = __iptunnel.delete_resource(nitro, object_name=__iptunnel.get_name())
                return nsresponse

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all iptunnel resources that are configured on netscaler.
                """
                __url = nitro.get_url() + NSIPTunnel.get_resourcetype()
                __json_iptunnels = nitro.get(__url).get_response_field(NSIPTunnel.get_resourcetype())
                __iptunnels = []
                for json_iptunnel in __json_iptunnels:
                        __iptunnels.append(NSIPTunnel(json_iptunnel))
                return __iptunnels



from nsbaseresource import NSBaseResource

__author__ = 'zojoncj'

class NSSystemCMDPolicy(NSBaseResource):

        # General Netscaler configuration object

        def __init__(self, json_data = None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSSystemCMDPolicy, self).__init__()

                self.options = {
                        'policyname' : '',
                        'action' : '',
                        'cmdspec' : '',
                }

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key]=json_data[key]

                self.resourcetype = NSSystemCMDPolicy.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "systemcmdpolicy"

        def set_policyname(self, policyname):
                self.options['policyname'] = policyname

        def get_policyname(self):
                return self.options['policyname']

        def set_action(self, action):
                self.options['action'] = action

        def get_action(self):
                return self.options['action']
        
        def set_cmdspec(self, cmdspec):
                self.options['cmdspec'] = cmdspec

        def get_cmdspec(self):
                return self.options['cmdspec']

        @staticmethod
        def add(nitro, systemcmdpolicy):
                __systemcmdpolicy = NSSystemCMDPolicy()
                __systemcmdpolicy.set_policyname(systemcmdpolicy.get_policyname())
                __systemcmdpolicy.set_action(systemcmdpolicy.get_action())
                __systemcmdpolicy.set_cmdspec(systemcmdpolicy.get_cmdspec())
                return __systemcmdpolicy.add_resource(nitro)

        @staticmethod
        def update(nitro, systemcmdpolicy):
                __systemcmdpolicy = NSSystemCMDPolicy()
                __systemcmdpolicy.set_policyname(systemcmdpolicy.get_policyname())
                __systemcmdpolicy.set_action(systemcmdpolicy.get_action())
                __systemcmdpolicy.set_cmdspec(systemcmdpolicy.get_cmdspec())
                return __systemcmdpolicy.update_resource(nitro)

        @staticmethod
        def delete(nitro, systemcmdpolicy):
                __systemcmdpolicy = NSSystemCMDPolicy()
                __systemcmdpolicy.set_policyname(systemcmdpolicy.get_policyname())
                nsresponse = __systemcmdpolicy.delete_resource(nitro, object_name=__systemcmdpolicy.get_policyname())
                return nsresponse

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all cmdpolicy resources that are configured on netscaler.
                """
                __url = nitro.get_url() + NSSystemCMDPolicy.get_resourcetype()
                __json_systemcmdpolicies = nitro.get(__url).get_response_field(NSSystemCMDPolicy.get_resourcetype())
                __systemcmdpolicies = []
                for json_systemcmdpolicy in __json_systemcmdpolicies:
                        __systemcmdpolicies.append(NSSystemCMDPolicy(json_systemcmdpolicy))
                return __systemcmdpolicies




import json
from nsexceptions import *


class NSNitroResponse:
        """ Generic class for accessing LB response dictionary. Can provide string response back and a parsed dictionary """
        __jresponse = False
        __sresponse = False
        errorcode = -1
        message = False
        failed = False

        def __init__(self, response):
                """ Constructor. reponse - string response """
                self.__sresponse = response
                self.__jresponse = json.loads(response)
                self.__parse_response()

        def get_json_response(self):
                """ Returns LB response as parsed dictionary """
                return self.__jresponse

        def get_string_response(self):
                """ Returns LB response as a string """
                return self.__sresponse

        def __parse_response(self):
                self.errorcode = self.__jresponse['errorcode']
                self.message   = self.__jresponse['message']
                if self.errorcode != 0:
                        self._raise_exception()
                        self.failed = True

        def get_response_field(self, field_name):
                """ Returns field_name of parsed JSON dictionary """
                if field_name in self.__jresponse:
                    return self.__jresponse[field_name]
                else:
                    return []

        def _raise_exception(self):
                """
                    Lookup specific exception based on errorcode in response,
                    if that fails raises NSNitroError.
                """
                exception_class = NSNitroExceptionClassMap[self.errorcode]
                raise exception_class(self.message, self.errorcode)



from nsbaseresource import NSBaseResource

class NSAcls(NSBaseResource):

        def __init__(self):

            super(NSAcls, self).__init__()

            self.options = {}
            self.resourcetype = NSAcls.get_resourcetype()


        @staticmethod
        def get_resourcetype():
            return "nsacls"

        @staticmethod
        def renumber(nitro):
            """
            Use this API to renumber nsacls.
            """
            __nsacls = NSAcls()
            return __nsacls.perform_operation(nitro, "renumber")

        @staticmethod
        def clear(nitro):
            """
            Use this API to clear nsacls.
            """
            __nsacls = NSAcls()
            return __nsacls.perform_operation(nitro, "clear")

        @staticmethod
        def apply(nitro):
            """
            Use this API to apply nsacls.
            """
            __nsacls = NSAcls()
            return __nsacls.perform_operation(nitro, "apply")



from nsbaseresource import NSBaseResource
__author__ = 'vlazarenko'

class NSRewritePolicyCSVServerBinding(NSBaseResource):

        # Binding class showing the csvserver that can be bound to rewritepolicy

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSRewritePolicyCSVServerBinding, self).__init__()
                self.options = {
                        'boundto' : '',
                        'priority' : '',
                        'activepolicy' : '',
                        'gotopriorityexpression' : '',
                        'labeltype' : '',
                        'labelname' : '',
                        'name' : '',
                }

                self.resourcetype = NSRewritePolicyCSVServerBinding.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key] = json_data[key]


        @staticmethod
        def get_resourcetype():
                return "rewritepolicy_csvserver_binding"

        def set_boundto(self, boundto):
                """
                Location where policy is bound.

                Default value: 0
                """
                self.options['boundto'] = boundto

        def get_boundto(self):
                """
                Location where policy is bound.

                Default value: 0
                """
                return self.options['boundto']

        def get_priority(self):
                """
                Specifies the priority of the policy.

                Default value: 0
                """
                return self.options['priority']

        def get_activepolicy(self):
                """
                Indicates whether policy is bound or not.

                Default value: 0
                """
                return self.options['activepolicy']

        def get_gotopriorityexpression(self):
                """
                Expression specifying the priority of the next policy which will get evaluated if the current policy
                rule evaluates to TRUE.

                Default value: 0
                """
                return self.options['gotopriorityexpression']

        def get_labeltype(self):
                """
                Type of policy label invocation.

                Default value: 0
                """
                return self.options['labeltype']

        def get_labelname(self):
                """
                Name of the label to invoke if the current policy rule evaluates to TRUE.

                Default value: 0
                """
                return self.options['labelname']

        def set_name(self, name):
                """
                Name of the rewrite policy.

                Default value: 0
                """
                self.options['name'] = name

        def get_name(self):
                """
                Name of the rewrite policy.

                Default value: 0
                """
                return self.options['name']

        @staticmethod
        def get(nitro, rewritepolicy_csvserver_binding):
                """
                Use this API to fetch all configured rewritepolicy_csvserver_binding resources.
                """
                __url = nitro.get_url() + NSRewritePolicyCSVServerBinding.get_resourcetype() + "/" + rewritepolicy_csvserver_binding.get_name()
                __json_resources = nitro.get(__url).get_response_field(NSRewritePolicyCSVServerBinding.get_resourcetype())
                __resources = []
                for json_resource in __json_resources:
                        __resources.append(NSRewritePolicyCSVServerBinding(json_resource))
                return __resources


from nsbaseresource import NSBaseResource

__author__ = 'ivanxx@gmail.com'

class SNMPCommunity(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(SNMPCommunity, self).__init__()

                self.options = {'communityname': '',
                                'permissions': ''}

                self.resourcetype = SNMPCommunity.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options:
                                        self.options[key] = json_data[key]

        @staticmethod
        def get_resourcetype():
                return "snmpcommunity"

        def set_communityname(self, communityname):
            self.options['communityname'] = communityname

        def set_permissions(self, permissions):
            self.options['permissions'] = permissions

        def get_communityname(self):
            return self.options['communityname']

        def get_permissions(self):
            return self.options['permissions']

        @staticmethod
        def add(nitro, snmpcommunity):
            __snmpcommunity = SNMPCommunity()
            __snmpcommunity.set_communityname(snmpcommunity.get_communityname())
            __snmpcommunity.set_permissions(snmpcommunity.get_permissions())
            return __snmpcommunity.add_resource(nitro)

        @staticmethod
        def delete(nitro, snmpcommunity):
            __snmpcommunity = SNMPCommunity()
            __snmpcommunity.set_communityname(snmpcommunity.get_communityname())
            return __snmpcommunity.delete_resource(nitro)

        @staticmethod
        def get(nitro, snmpcommunity):
            __snmpcommunity = SNMPCommunity()
            __snmpcommunity.set_communityname(snmpcommunity)
            __snmpcommunity.get_resource(nitro, object_name=__snmpcommunity.get_communityname())
            return __snmpcommunity

        @staticmethod
        def get_all(nitro):
            __url = nitro.get_url() + SNMPCommunity.get_resourcetype()
            __json_snmpcommunities = nitro.get(__url).get_response_field(SNMPCommunity.get_resourcetype())
            __snmpcommunities = []
            for json_snmpcommunity in __json_snmpcommunities:
                __snmpcommunities.append(SNMPCommunity(json_snmpcommunity))
            return __snmpcommunities



from nsnitro.nsutil import NSNitroError
import urllib
import json


class NSBaseResource(object):

        options = {}
        resourcetype = False

        def __init__(self):
                self.options = {}
                self.resourcetype = False
                self.__baseaction = False

        def __str__(self):
                ret = ""
                for key, value in self.options.items():
                        ret += "\t%s: \t\t%s\n" % (key, value)

                return ret

        def set_action(self, action):
                self.__baseaction = action

        def set_options(self, options):
                self.options = options

                # Filter out empty options
                self.options = dict([(k, v) for k, v in self.options.items() if v])

        def get_payload(self):
                options = dict([(k, v) for k, v in self.options.items() if v])
                if self.__baseaction:
                        payload = {"object": json.dumps({"params": {"action": self.__baseaction}, self.resourcetype: options})}
                else:
                        payload = {"object": json.dumps({self.resourcetype: options})}
                #print payload
                return payload

        def get_put_payload(self, sessionid):
                options = dict([(k, v) for k, v in self.options.items() if v])
                if self.__baseaction:
                        payload = {"params": {"action": self.__baseaction}, self.resourcetype: options}
                else:
                        payload = {"sessionid": sessionid, self.resourcetype: options}
                #print payload
                return payload

        def get_delete_args(self):
                options = dict([(k, v) for k, v in self.options.items() if v])

                args = "?args="

                for key, value in options.iteritems():
                        args = "%s%s:%s%s" % (args, key, urllib.quote_plus(value) if type(value) is str else value, ",")

                args = args[:-1]  # remove last comma

                return args

        def perform_operation(self, nitro, action):
                self.set_action(action)
                response = nitro.post(self.get_payload())
                return response

        def get_resource(self, nitro, object_name=None):
                url = "%s%s/%s" % (nitro.get_url(),
                                   self.resourcetype,
                                   object_name if object_name else self.options['name'])
                response = nitro.get(url)

                if response.failed:
                        raise NSNitroError(response.message)

                for resource in response.get_response_field(self.resourcetype):
                                for k in resource.iterkeys():
                                        self.options[k] = resource[k]

        def add_resource(self, nitro):
                response = nitro.post(self.get_payload())
                return response

        def update_resource(self, nitro):
                response = nitro.put(self.get_put_payload(nitro.get_sessionid()))
                if response.failed:
                        raise NSNitroError(response.message)
                return response

        def delete_resource(self, nitro, object_name=None):
                url = "%s%s" % (nitro.get_url(), self.resourcetype)

                if object_name is not None:
                    url += "/%s" % object_name
                elif 'name' in self.options and self.options['name'] is not None:
                    url += "/%s" % self.options['name']

                urlargs = self.get_delete_args()
                url += urlargs

                response = nitro.delete(url)

                if response.failed:
                        raise NSNitroError(response.message)

                return response



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSIP(NSBaseResource):

        # General Netscaler configuration object

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSIP, self).__init__()

                self.options = {'ipaddress': '',
                                'netmask': '',
                                'type': '',
                                'arp': '',
                                'icmp': '',
                                'vserver': '',
                                'telnet': '',
                                'ftp': '',
                                'gui': '',
                                'ssh': '',
                                'snmp': '',
                                'mgmtaccess': '',
                                'restrictaccess': '',
                                'dynamicrouting': '',
                                'ospf': '',
                                'bgp': '',
                                'rip': '',
                                'hostroute': '',
                                'hostrtgw': '',
                                'metric': '',
                                'vserverrhilevel': '',
                                'ospflsatype': '',
                                'ospfarea': '',
                                'state': '',
                                'vrid': '',
                                'flags': '',
                                'ospfareaval': '',
                                'viprtadv2bsd': '',
                                'vipvsercount': '',
                                'vipvserdowncount': '',
                                'freeports': '',
                                'iptype': '',
                                'count': ''}

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

                self.resourcetype = NSIP.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "nsip"

        def set_ipaddress(self, ipaddress):
                self.options['ipaddress'] = ipaddress

        def get_ipaddress(self):
                return self.options['ipaddress']

        def set_netmask(self, netmask):
                self.options['netmask'] = netmask

        def get_netmask(self):
                return self.options['netmask']

        def set_type(self, mytype):
                self.options['type'] = mytype

        def get_type(self):
                return self.options['type']

        def set_arp(self, arp):
                self.options['arp'] = arp

        def get_arp(self):
                return self.options['arp']

        def set_icmp(self, icmp):
                self.options['icmp'] = icmp

        def get_icmp(self):
                return self.options['icmp']

        def set_vserver(self, vserver):
                self.options['vserver'] = vserver

        def get_vserver(self):
                return self.options['vserver']

        def set_telnet(self, telnet):
                self.options['telnet'] = telnet

        def get_telnet(self):
                return self.options['telnet']

        def set_ftp(self, ftp):
                self.options['ftp'] = ftp

        def get_ftp(self):
                return self.options['ftp']

        def set_gui(self, gui):
                self.options['gui'] = gui

        def get_gui(self):
                return self.options['gui']

        def set_ssh(self, ssh):
                self.options['ssh'] = ssh

        def get_ssh(self):
                return self.options['ssh']

        def set_snmp(self, snmp):
                self.options['snmp'] = snmp

        def get_snmp(self):
                return self.options['snmp']

        def set_mgmtaccess(self, mgmtaccess):
                self.options['mgmtaccess'] = mgmtaccess

        def get_mgmtaccess(self):
                return self.options['mgmtaccess']

        def set_restrictaccess(self, restrictaccess):
                self.options['restrictaccess'] = restrictaccess

        def get_restrictaccess(self):
                return self.options['restrictaccess']

        def set_dynamicrouting(self, dynamicrouting):
                self.options['dynamicrouting'] = dynamicrouting

        def get_dynamicrouting(self):
                return self.options['dynamicrouting']

        def set_ospf(self, ospf):
                self.options['ospf'] = ospf

        def get_ospf(self):
                return self.options['ospf']

        def set_bgp(self, bgp):
                self.options['bgp'] = bgp

        def get_bgp(self):
                return self.options['bgp']

        def set_rip(self, rip):
                self.options['rip'] = rip

        def get_rip(self):
                return self.options['rip']

        def set_hostroute(self, hostroute):
                self.options['hostroute'] = hostroute

        def get_hostroute(self):
                return self.options['hostroute']

        def set_hostrtgw(self, hostrtgw):
                self.options['hostrtgw'] = hostrtgw

        def get_hostrtgw(self):
                return self.options['hostrtgw']

        def set_metric(self, metric):
                self.options['metric'] = metric

        def get_metric(self):
                return self.options['metric']

        def set_vserverrhilevel(self, vserverrhilevel):
                self.options['vserverrhilevel'] = vserverrhilevel

        def get_vserverrhilevel(self):
                return self.options['vserverrhilevel']

        def set_ospflsatype(self, ospflsatype):
                self.options['ospflsatype'] = ospflsatype

        def get_ospflsatype(self):
                return self.options['ospflsatype']

        def set_ospfarea(self, ospfarea):
                self.options['ospfarea'] = ospfarea

        def get_ospfarea(self):
                return self.options['ospfarea']

        def set_state(self, state):
                self.options['state'] = state

        def get_state(self):
                return self.options['state']

        def set_vrid(self, vrid):
                self.options['vrid'] = vrid

        def get_vrid(self):
                return self.options['vrid']

        def get_flags(self):
                return self.options['flags']

        def get_ospfareaval(self):
                return self.options['ospfareaval']

        def get_viprtadv2bsd(self):
                return self.options['viprtadv2bsd']

        def get_vipvsercount(self):
                return self.options['vipvsercount']

        def get_vipvserdowncount(self):
                return self.options['vipvserdowncount']

        def get_freeports(self):
                return self.options['freeports']

        def get_iptype(self):
                return self.options['iptype']

        def get_count(self):
                return self.options['count']

        @staticmethod
        def add(nitro, ip):
                __ip = NSIP()
                __ip.set_ipaddress(ip.get_ipaddress())
                __ip.set_netmask(ip.get_netmask())
                __ip.set_type(ip.get_type())
                __ip.set_arp(ip.get_arp())
                __ip.set_icmp(ip.get_icmp())
                __ip.set_vserver(ip.get_vserver())
                __ip.set_telnet(ip.get_telnet())
                __ip.set_ftp(ip.get_ftp())
                __ip.set_gui(ip.get_gui())
                __ip.set_ssh(ip.get_ssh())
                __ip.set_snmp(ip.get_snmp())
                __ip.set_mgmtaccess(ip.get_mgmtaccess())
                __ip.set_restrictaccess(ip.get_restrictaccess())
                __ip.set_dynamicrouting(ip.get_dynamicrouting())
                __ip.set_ospf(ip.get_ospf())
                __ip.set_bgp(ip.get_bgp())
                __ip.set_rip(ip.get_rip())
                __ip.set_hostroute(ip.get_hostroute())
                __ip.set_hostrtgw(ip.get_hostrtgw())
                __ip.set_metric(ip.get_metric())
                __ip.set_vserverrhilevel(ip.get_vserverrhilevel())
                __ip.set_ospflsatype(ip.get_ospflsatype())
                __ip.set_ospfarea(ip.get_ospfarea())
                __ip.set_state(ip.get_state())
                __ip.set_vrid(ip.get_vrid())
                return __ip.add_resource(nitro)

        @staticmethod
        def update(nitro, ip):
                __ip = NSIP()
                __ip.set_ipaddress(ip.get_ipaddress())
                __ip.set_netmask(ip.get_netmask())
                __ip.set_arp(ip.get_arp())
                __ip.set_icmp(ip.get_icmp())
                __ip.set_vserver(ip.get_vserver())
                __ip.set_telnet(ip.get_telnet())
                __ip.set_ftp(ip.get_ftp())
                __ip.set_gui(ip.get_gui())
                __ip.set_ssh(ip.get_ssh())
                __ip.set_snmp(ip.get_snmp())
                __ip.set_mgmtaccess(ip.get_mgmtaccess())
                __ip.set_restrictaccess(ip.get_restrictaccess())
                __ip.set_dynamicrouting(ip.get_dynamicrouting())
                __ip.set_ospf(ip.get_ospf())
                __ip.set_bgp(ip.get_bgp())
                __ip.set_rip(ip.get_rip())
                __ip.set_hostroute(ip.get_hostroute())
                __ip.set_hostrtgw(ip.get_hostrtgw())
                __ip.set_metric(ip.get_metric())
                __ip.set_vserverrhilevel(ip.get_vserverrhilevel())
                __ip.set_ospflsatype(ip.get_ospflsatype())
                __ip.set_ospfarea(ip.get_ospfarea())
                __ip.set_vrid(ip.get_vrid())
                return __ip.update_resource(nitro)

        @staticmethod
        def delete(nitro, ip):
                __ip = NSIP()
                __ip.set_ipaddress(ip.get_ipaddress())
                nsresponse = __ip.delete_resource(nitro, object_name=ip.get_ipaddress())
                return nsresponse

        @staticmethod
        def enable(nitro, ip):
                __ip = NSIP()
                __ip.set_ipaddress(ip.get_ipaddress())
                return __ip.perform_operation(nitro, "enable")

        @staticmethod
        def disable(nitro, ip):
                __ip = NSIP()
                __ip.set_ipaddress(ip.get_ipaddress())
                return __ip.perform_operation(nitro, "disable")

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all nsip resources that are configured on netscaler.
                """
                __url = nitro.get_url() + NSIP.get_resourcetype()
                __json_ips = nitro.get(__url).get_response_field(NSIP.get_resourcetype())
                __ips = []
                for json_ip in __json_ips:
                        __ips.append(NSIP(json_ip))
                return __ips



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSSSLCertKey(NSBaseResource):

        # General Netscaler configuration object

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSSSLCertKey, self).__init__()

                self.options = {'certkey': '',
                                'cert': '',
                                'key': '',
                                'password': '',
                                'fipskey': '',
                                'inform': '',
                                'passplain': '',
                                'expirymonitor': '',
                                'notificationperiod': '',
                                'linkcertkeyname': '',
                                'nodomaincheck': '',
                                'signaturealg': '',
                                'serial': '',
                                'issuer': '',
                                'clientcertnotbefore': '',
                                'clientcertnotafter': '',
                                'daystoexpiration': '',
                                'subject': '',
                                'publickey': '',
                                'publickeysize': '',
                                'version': '',
                                'priority': '',
                                'status': '',
                                'passcrypt': '',
                                'data': '',
                                'servicename': ''}

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

                self.resourcetype = NSSSLCertKey.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "sslcertkey"

        def set_certkey(self, certkey):
                self.options['certkey'] = certkey

        def get_certkey(self):
                return self.options['certkey']

        def set_cert(self, cert):
                self.options['cert'] = cert

        def get_cert(self):
                return self.options['cert']

        def set_key(self, key):
                self.options['key'] = key

        def get_key(self):
                return self.options['key']

        def set_password(self, password):
                self.options['password'] = password

        def get_password(self):
                return self.options['password']

        def set_fipskey(self, fipskey):
                self.options['fipskey'] = fipskey

        def get_fipskey(self):
                return self.options['fipskey']

        def set_inform(self, inform):
                self.options['inform'] = inform

        def get_inform(self):
                return self.options['inform']

        def set_passplain(self, passplain):
                self.options['passplain'] = passplain

        def get_passplain(self):
                return self.options['passplain']

        def set_expirymonitor(self, expirymonitor):
                self.options['expirymonitor'] = expirymonitor

        def get_expirymonitor(self):
                return self.options['expirymonitor']

        def set_notificationperiod(self, notificationperiod):
                self.options['notificationperiod'] = notificationperiod

        def get_notificationperiod(self):
                return self.options['notificationperiod']

        def set_linkcertkeyname(self, linkcertkeyname):
                self.options['linkcertkeyname'] = linkcertkeyname

        def get_linkcertkeyname(self):
                return self.options['linkcertkeyname']

        def set_nodomaincheck(self, nodomaincheck):
                self.options['nodomaincheck'] = nodomaincheck

        def get_nodomaincheck(self):
                return self.options['nodomaincheck']

        def set_passcrypt(self, passcrypt):
                self.options['passcrypt'] = passcrypt

        def get_passcrypt(self):
                return self.options['passcrypt']

        def get_signaturealg(self):
                return self.options['signaturealg']

        def get_serial(self):
                return self.options['serial']

        def get_issuer(self):
                return self.options['issuer']

        def get_clientcertnotbefore(self):
                return self.options['clientcertnotbefore']

        def get_clientcertnotafter(self):
                return self.options['clientcertnotafter']

        def get_daystoexpiration(self):
                return self.options['daystoexpiration']

        def get_subject(self):
                return self.options['subject']

        def get_publickey(self):
                return self.options['publickey']

        def get_publickeysize(self):
                return self.options['publickeysize']

        def get_version(self):
                return self.options['version']

        def get_priority(self):
                return self.options['priority']

        def get_status(self):
                return self.options['status']

        def get_data(self):
                return self.options['data']

        def get_servicename(self):
                return self.options['servicename']

        @staticmethod
        def get(nitro, resource):
                """
                Use this API to fetch SSL certkey resource of given name.
                """
                __resource = NSSSLCertKey()
                __resource.get_resource(nitro, resource.get_certkey())
                return __resource

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all configured SSL certkey resources.
                """
                __url = nitro.get_url() + NSSSLCertKey.get_resourcetype()
                __json_resources = nitro.get(__url).get_response_field(NSSSLCertKey.get_resourcetype())
                __resources = []
                for json_resource in __json_resources:
                        __resources.append(NSSSLCertKey(json_resource))
                return __resources

        @staticmethod
        def add(nitro, resource):
                """
                Use this API to add resource.
                """
                __resource = NSSSLCertKey()
                __resource.set_certkey(resource.get_certkey())
                __resource.set_cert(resource.get_cert())
                __resource.set_key(resource.get_key())
                __resource.set_password(resource.get_password())
                __resource.set_fipskey(resource.get_fipskey())
                __resource.set_inform(resource.get_inform())
                __resource.set_passplain(resource.get_passplain())
                __resource.set_passcrypt(resource.get_passcrypt())
                __resource.set_expirymonitor(resource.get_expirymonitor())
                __resource.set_notificationperiod(resource.get_notificationperiod())
                return __resource.add_resource(nitro)

        @staticmethod
        def delete(nitro, resource):
                """
                Use this API to delete server of a given name.
                """
                __resource = NSSSLCertKey()
                __resource.set_certkey(resource.get_certkey())
                nsresponse = __resource.delete_resource(nitro)
                return nsresponse

        @staticmethod
        def update(nitro, resource):
                """
                Use this API to update a server of a given name.
                """
                __resource = NSSSLCertKey()
                __resource.set_certkey(resource.get_certkey())
                __resource.set_expirymonitor(resource.get_expirymonitor())
                __resource.set_notificationperiod(resource.get_notificationperiod())
                return __resource.update_resource(nitro)

        @staticmethod
        def link(nitro, resource):
                """
                Use this API to link resource of a given name
                """
                __resource = NSSSLCertKey()
                __resource.set_certkey(resource.get_certkey())
                __resource.set_linkcertkeyname(resource.get_linkcertkeyname())
                nsresponse = __resource.perform_operation(nitro, "link")
                return nsresponse

        @staticmethod
        def unlink(nitro, resource):
                """
                Use this API to link resource of a given name
                """
                __resource = NSSSLCertKey()
                __resource.set_certkey(resource.get_certkey())
                nsresponse = __resource.perform_operation(nitro, "unlink")
                return nsresponse



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'

class NSRewriteAction(NSBaseResource):

        # Configuration for content-switching rewrite action resource.

        def __init__(self, json_data = None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSRewriteAction, self).__init__()

                self.options = {
                        'name' : '',
                        'type' : '',
                        'target' : '',
                        'stringbuilderexpr' : '',
                        'pattern' : '',
                        'search' : '',
                        'allow_unsafe_pi' : '',
                        'refinesearch' : '',
                        'comment' : '',
                        'allow_unsafe_pi1' : '',
                        'newname' : '',
                        'hits' : '',
                        'undefhits' : '',
                        'referencecount' : '',
                        'description' : '',
                }

                self.resourcetype = NSRewriteAction.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key] = json_data[key]


        @staticmethod
        def get_resourcetype():
                return "rewriteaction"

        def set_name(self, name):
                self.options['name'] = name

        def get_name(self):
                return self.options['name']
        
        def set_type(self, type):
                self.options['type'] = type
        
        def get_type(self):
                return self.options['type']
        
        def set_target(self, target):
                self.options['target'] = target
        
        def get_target(self):
                return self.options['target']
        
        def set_stringbuilderexpr(self, stringbuilderexpr):
                self.options['stringbuilderexpr'] = stringbuilderexpr
        
        def get_stringbuilderexpr(self):
                return self.options['stringbuilderexpr']
        
        def set_pattern(self, pattern):
                self.options['pattern'] = pattern
        
        def get_pattern(self):
                return self.options['pattern']
        
        def set_search(self, search):
                self.options['search'] = search
        
        def get_search(self):
                return self.options['search']
        
        def set_allow_unsafe_pi(self, allow_unsafe_pi):
                self.options['allow_unsafe_pi'] = allow_unsafe_pi
        
        def get_allow_unsafe_pi(self):
                return self.options['allow_unsafe_pi']
        
        def set_refinesearch(self, refinesearch):
                self.options['refinesearch'] = refinesearch
        
        def get_refinesearch(self):
                return self.options['refinesearch']
        
        def set_comment(self, comment):
                self.options['comment'] = comment
        
        def get_comment(self):
                return self.options['comment']
        
        def set_allow_unsafe_pi1(self, allow_unsafe_pi1):
                self.options['allow_unsafe_pi1'] = allow_unsafe_pi1
        
        def get_allow_unsafe_pi1(self):
                return self.options['allow_unsafe_pi1']
        
        def set_newname(self, newname):
                self.options['newname'] = newname
        
        def get_newname(self):
                return self.options['newname']
        
        def get_hits(self):
                return self.options['hits']
        
        def get_undefhits(self):
                return self.options['undefhits']
        
        def get_referencecount(self):
                return self.options['referencecount']
        
        def get_description(self):
                return self.options['description']


        # Operations methods
        @staticmethod
        def get(nitro, rewriteaction):
                """
                Use this API to fetch rewriteaction resource of given name.
                """
                __rewriteaction = NSRewriteAction()
                __rewriteaction.set_name(rewriteaction.get_name())
                __rewriteaction.get_resource(nitro)
                return __rewriteaction

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all configured rewriteaction resources.
                """
                __url = nitro.get_url() + NSRewriteAction.get_resourcetype()
                __json_cspolicies = nitro.get(__url).get_response_field(NSRewriteAction.get_resourcetype())
                __rewritepolicies = []
                for json_rewriteaction in __json_cspolicies:
                        __rewritepolicies.append(NSRewriteAction(json_rewriteaction))
                return __rewritepolicies

        @staticmethod
        def add(nitro, rewriteaction):
                """
                Use this API to add rewriteaction.
                """
                __rewriteaction = NSRewriteAction()
                __rewriteaction.set_name(rewriteaction.get_name())
                __rewriteaction.set_type(rewriteaction.get_type())
                __rewriteaction.set_target(rewriteaction.get_target())
                __rewriteaction.set_stringbuilderexpr(rewriteaction.get_stringbuilderexpr())
                __rewriteaction.set_pattern(rewriteaction.get_pattern())
                __rewriteaction.set_search(rewriteaction.get_search())
                __rewriteaction.set_allow_unsafe_pi(rewriteaction.get_allow_unsafe_pi())
                __rewriteaction.set_refinesearch(rewriteaction.get_refinesearch())
                __rewriteaction.set_comment(rewriteaction.get_comment())

                return __rewriteaction.add_resource(nitro)


        @staticmethod
        def delete(nitro, rewriteaction):
                """
                Use this API to delete rewriteaction of a given name.
                """
                __rewriteaction = NSRewriteAction()
                __rewriteaction.set_name(rewriteaction.get_name())
                nsresponse = __rewriteaction.delete_resource(nitro)
                return nsresponse

        @staticmethod
        def update(nitro, rewriteaction):
                """
                Use this API to update a rewriteaction of a given name.
                """
                __rewriteaction = NSRewriteAction()
                __rewriteaction.set_name(rewriteaction.get_name())
                __rewriteaction.set_target(rewriteaction.get_target())
                __rewriteaction.set_stringbuilderexpr(rewriteaction.get_stringbuilderexpr())
                __rewriteaction.set_pattern(rewriteaction.get_pattern())
                __rewriteaction.set_search(rewriteaction.get_search())
                __rewriteaction.set_allow_unsafe_pi(rewriteaction.get_allow_unsafe_pi())
                __rewriteaction.set_allow_unsafe_pi1(rewriteaction.get_allow_unsafe_pi1())
                __rewriteaction.set_refinesearch(rewriteaction.get_refinesearch())
                __rewriteaction.set_comment(rewriteaction.get_comment())
                return __rewriteaction.update_resource(nitro)


        # No unset functionality for now.

        @staticmethod
        def rename(nitro, rewriteaction):
                """
                Use this API to rename rewriteaction.
                """
                __rewriteaction = NSRewriteAction()
                __rewriteaction.set_name(rewriteaction.get_name())
                __rewriteaction.set_newname(rewriteaction.get_newname())
                return __rewriteaction.perform_operation(nitro, "rename")



from nsbaseresource import NSBaseResource


class NSServiceGroupServerBinding(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSServiceGroupServerBinding, self).__init__()
                self.options = {
                        'servicegroupname' : '',
                        'ip' : '',
                        'servername' : '',
                        'port' : '',
                        'weight' : '',
                        'state' : '',
                        'serverid' : '',
                        'hashid' : '',
                        'svrstate': ''
                }

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key]=json_data[key]

                self.resourcetype = NSServiceGroupServerBinding.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "servicegroup_servicegroupmember_binding"

        # Getters and setters for configurable options
        def set_ip(self, ip):
                """
                The IP address of the server that needs to be binded.
                Default value: 0
                """
                self.options['ip'] = ip

        def get_ip(self):
                """
                The IP address of the server.
                Default value: 0
                """
                return self.options['ip']

        def set_port(self, port):
                """
                The port of the server.
                """
                self.options['port'] = port

        def get_port(self):
                """
                The port of the server that its binded to.
                """
                return self.options['port']

        def set_state(self, state):
                """
                The state of the server.
                """
                self.options['state'] = state

        def get_state(self):
                """
                The state of the server that its binded to.
                """
                return self.options['state']

        def get_svrstate(self):
                """
                The svrstate of the server that its binded to.
                """
                return self.options['svrstate']

        def set_weight(self, weight):
                """
                The weight for the specified service.
                Default value: 0
                Minimum value =  1
                Maximum value =  100
                """
                self.options['weight'] = weight

        def get_weight(self):
                """
                The weight for the specified service.
                Default value: 0
                Minimum value =  1
                Maximum value =  100
                """
                return self.options['weight']

        def set_hashid(self, hashid):
                """
                hash Id.
                Default value: 0
                """
                self.options['hashid'] = hashid

        def get_hashid(self):
                """
                hash Id.
                Default value: 0
                """
                return self.options['hashid']

        def set_serverid(self, serverid):
                """
                server Id.
                Default value: 0
                """
                self.options['serverid'] = serverid

        def get_serverid(self):
                """
                server Id.
                Default value: 0
                """
                return self.options['serverid']

        def set_servername(self, servername):
                """
                The server name to which the service is bound.
                """
                self.options['servername'] = servername

        def get_servername(self):
                """
                The server name to which the service is bound.
                """
                return self.options['servername']

        def set_servicegroupname(self, servicegroupname):
                """
                The name of the service group that is bound.
                """
                self.options['servicegroupname'] = servicegroupname

        def get_servicegroupname(self):
                """
                The name of the service group that is bound.
                """
                return self.options['servicegroupname']


        # Operations methods
        @staticmethod
        def get(nitro, servicegroup_server_binding):
                """
                Use this API to fetch configured servicegroup_server_binding
                """
                __url = nitro.get_url() + NSServiceGroupServerBinding.get_resourcetype() + "/" + servicegroup_server_binding.get_servicegroupname()
                __json_services = nitro.get(__url).get_response_field(NSServiceGroupServerBinding.get_resourcetype())
                __services = []
                for json_service in __json_services:
                        __services.append(NSServiceGroupServerBinding(json_service))
                return __services

        @staticmethod
        def get_server(nitro, servicegroup_server_binding):
                """
                Use this API to fetch a server configured to a servicegroup_server_binding
                """
                __servicegroup_server_binding = NSServiceGroupServerBinding()
                __servicegroup_server_binding.set_servicegroupname(servicegroup_server_binding.get_servicegroupname())
                __servicegroup_server_binding.set_servername(servicegroup_server_binding.get_servername())
                __filter = servicegroup_server_binding.get_servicegroupname() + "?filter=servername:" + servicegroup_server_binding.get_servername()
                __servicegroup_server_binding.get_resource(nitro, __filter)
                return __servicegroup_server_binding


        @staticmethod
        def add(nitro, servicegroup_server_binding):
                """
                Use this API to add a server to servergroup through servicegroup_server_binding.
                """
                __servicegroup_server_binding = NSServiceGroupServerBinding()
                __servicegroup_server_binding.set_servername(servicegroup_server_binding.get_servername())
                __servicegroup_server_binding.set_ip(servicegroup_server_binding.get_ip())
                __servicegroup_server_binding.set_port(servicegroup_server_binding.get_port())
                __servicegroup_server_binding.set_servicegroupname(servicegroup_server_binding.get_servicegroupname())
                __servicegroup_server_binding.set_weight(servicegroup_server_binding.get_weight())
                __servicegroup_server_binding.set_serverid(servicegroup_server_binding.get_serverid())
                __servicegroup_server_binding.set_state(servicegroup_server_binding.get_state())
                __servicegroup_server_binding.set_hashid(servicegroup_server_binding.get_hashid())
                return __servicegroup_server_binding.update_resource(nitro)

        @staticmethod
        def delete(nitro, servicegroup_server_binding):
                """
                Use this API to delete a server from servicegroup_server_binding of a given name.
                """
                __servicegroup_server_binding = NSServiceGroupServerBinding()
                __servicegroup_server_binding.set_servicegroupname(servicegroup_server_binding.get_servicegroupname())
                __servicegroup_server_binding.set_servername(servicegroup_server_binding.get_servername())
                __servicegroup_server_binding.set_ip(servicegroup_server_binding.get_ip())
                __servicegroup_server_binding.set_port(servicegroup_server_binding.get_port())
                __filter = servicegroup_server_binding.get_servicegroupname() + "/" + servicegroup_server_binding.get_servername()
                return __servicegroup_server_binding.delete_resource(nitro, __filter)



from nsbaseresource import NSBaseResource
__author__ = 'vlazarenko'


class NSCSVServerRewritePolicyBinding(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSCSVServerRewritePolicyBinding, self).__init__()
                self.options = {'policyname': '',
                                'priority': '',
                                'gotopriorityexpression': '',
                                'bindpoint': '',
                                'invoke': '',
                                'labeltype': '',
                                'labelname': '',
                                'name': ''}

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

                self.resourcetype = NSCSVServerRewritePolicyBinding.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "csvserver_rewritepolicy_binding"

        def set_policyname(self, policyname):
                """
                Bound classic (Compression/Authorization/Filter/Audit) policies to this vserver.

                Default value: 0
                """
                self.options['policyname'] = policyname

        def get_policyname(self):
                """
                Bound classic (Compression/Authorization/Filter/Audit) policies to this vserver.

                Default value: 0
                """
                return self.options['policyname']

        def set_priority(self, priority):
                """
                Priority for the policy.

                Default value: 0
                """
                self.options['priority'] = priority

        def get_priority(self):
                """
                Priority for the policy.

                Default value: 0
                """
                return self.options['priority']

        def set_gotopriorityexpression(self, gotopriorityexpression):
                """
                Expression specifying the priority of the next policy
                which will get evaluated if the current policy rule
                evaluates to TRUE.

                Default value: 0
                """
                self.options['gotopriorityexpression'] = gotopriorityexpression

        def get_gotopriorityexpression(self):
                """
                Expression specifying the priority of the next policy
                which will get evaluated if the current policy rule
                evaluates to TRUE.

                Default value: 0
                """
                return self.options['gotopriorityexpression']

        def set_bindpoint(self, bindpoint):
                """
                The bindpoint to which the policy is bound.

                Default value: 0
                """
                self.options['bindpoint'] = bindpoint

        def get_bindpoint(self):
                """
                The bindpoint to which the policy is bound.

                Default value: 0
                """
                return self.options['bindpoint']

        def set_invoke(self, invoke):
                """
                Invoke flag.

                Default value: 0
                """
                self.options['invoke'] = invoke

        def get_invoke(self):
                """
                Invoke flag.

                Default value: 0
                """
                return self.options['invoke']

        def set_labeltype(self, labeltype):
                """
                The invocation type.

                Default value: 0
                """
                self.options['labeltype'] = labeltype

        def get_labeltype(self):
                """
                The invocation type.

                Default value: 0
                """
                return self.options['labeltype']

        def set_labelname(self, labelname):
                """
                Name of the label invoked.

                Default value: 0
                """
                self.options['labelname'] = labelname

        def get_labelname(self):
                """
                Name of the label invoked.

                Default value: 0
                """
                return self.options['labelname']

        def set_name(self, name):
                """
                The virtual server name (created with the add cs vserver or
                add cr vserver command) for which the content
                switching policy will be set.

                Default value: 0
                Minimum length =  1.
                """
                self.options['name'] = name

        def get_name(self):
                """
                The virtual server name (created with the add cs vserver or
                add cr vserver command) for which the content
                switching policy will be set.

                Default value: 0
                Minimum length =  1.
                """
                return self.options['name']

        # Operations methods
        @staticmethod
        def get(nitro, csvserver_rewritepolicy_binding):
                """
                Use this API to fetch all configured csvserver_rewritepolicy_binding resources.
                """
                __url = nitro.get_url() + NSCSVServerRewritePolicyBinding.get_resourcetype() + "/" + csvserver_rewritepolicy_binding.get_name()
                __json_resources = nitro.get(__url).get_response_field(NSCSVServerRewritePolicyBinding.get_resourcetype())
                __resources = []
                for json_resource in __json_resources:
                        __resources.append(NSCSVServerRewritePolicyBinding(json_resource))
                return __resources


        @staticmethod
        def add(nitro, csvserver_rewritepolicy_binding):
                """
                Use this API to add csvserver_rewritepolicy_binding.
                """
                __csvserver_rewritepolicy_binding = NSCSVServerRewritePolicyBinding()
                __csvserver_rewritepolicy_binding.set_name(csvserver_rewritepolicy_binding.get_name())
                __csvserver_rewritepolicy_binding.set_policyname(csvserver_rewritepolicy_binding.get_policyname())
                __csvserver_rewritepolicy_binding.set_priority(csvserver_rewritepolicy_binding.get_priority())
                __csvserver_rewritepolicy_binding.set_gotopriorityexpression(csvserver_rewritepolicy_binding.get_gotopriorityexpression())
                __csvserver_rewritepolicy_binding.set_bindpoint(csvserver_rewritepolicy_binding.get_bindpoint())
                __csvserver_rewritepolicy_binding.set_invoke(csvserver_rewritepolicy_binding.get_invoke())
                __csvserver_rewritepolicy_binding.set_labeltype(csvserver_rewritepolicy_binding.get_labeltype())
                __csvserver_rewritepolicy_binding.set_labelname(csvserver_rewritepolicy_binding.get_labelname())
                return __csvserver_rewritepolicy_binding.update_resource(nitro)

        @staticmethod
        def delete(nitro, csvserver_rewritepolicy_binding):
                """
                Use this API to delete csvserver_rewritepolicy_binding of a given name.
                """
                __csvserver_rewritepolicy_binding = NSCSVServerRewritePolicyBinding()
                __csvserver_rewritepolicy_binding.set_name(csvserver_rewritepolicy_binding.get_name())
                __csvserver_rewritepolicy_binding.set_policyname(csvserver_rewritepolicy_binding.get_policyname())
                __csvserver_rewritepolicy_binding.set_priority(csvserver_rewritepolicy_binding.get_priority())
                __csvserver_rewritepolicy_binding.set_bindpoint(csvserver_rewritepolicy_binding.get_bindpoint())
                nsresponse = __csvserver_rewritepolicy_binding.delete_resource(nitro)
                return nsresponse


from nsbaseresource import NSBaseResource

__author__ = 'ndenev@gmail.com'


class SystemGlobalAuthTacacsPolicyBinding(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(SystemGlobalAuthTacacsPolicyBinding, self).__init__()

                self.options = {'priority': '',
                                'builtin': '',
                                'policyname': ''}

                self.resourcetype = SystemGlobalAuthTacacsPolicyBinding.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options:
                                        self.options[key] = json_data[key]

        @staticmethod
        def get_resourcetype():
                return "systemglobal_authenticationtacacspolicy_binding"

        def set_priority(self, priority):
                """
                The priority of the command policy.
                """
                self.options['priority'] = priority

        def get_priority(self):
                return self.options['priority']

        def set_builtin(self, builtin):
                """
                Indicates that a variable is a built-in (SYSTEM INTERNAL) type.
                Possible values = MODIFIABLE, DELETABLE, IMMUTABLE
                """
                self.options['builtin'] = builtin

        def get_builtin(self):
                return self.options['builtin']

        def set_policyname(self, policyname):
                """
                The name of the command policy.
                """
                self.options['policyname'] = policyname

        def get_policyname(self):
                return self.options['policyname']

        # Operations methods
        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch the configured systemglobal_authenticationtacacspolicy_binding resources.
                """
                __url = nitro.get_url() + SystemGlobalAuthTacacsPolicyBinding.get_resourcetype()
                __json_policybindings = nitro.get(__url).get_response_field(SystemGlobalAuthTacacsPolicyBinding.get_resourcetype())
                __policybindings = []
                for json_policybinding in __json_policybindings:
                        __policybindings.append(SystemGlobalAuthTacacsPolicyBinding(json_policybinding))
                return __policybindings

        @staticmethod
        def add(nitro, polycibinding):
                """
                Use this API to add systemglobal_authenticationtacacspolicy_binding.
                """
                __polycibinding = SystemGlobalAuthTacacsPolicyBinding()
                __polycibinding.set_policyname(polycibinding.get_policyname())
                __polycibinding.set_priority(polycibinding.get_priority())
                __polycibinding.set_builtin(polycibinding.get_builtin())
                return __polycibinding.add_resource(nitro)

        @staticmethod
        def delete(nitro, policybinding):
                """
                Use this API to delete systemglobal_authenticationtacacspolicy_binding of a given name.
                """
                __policybinding = SystemGlobalAuthTacacsPolicyBinding()
                __policybinding.set_policyname(policybinding.get_policyname())
                nsresponse = __policybinding.delete_resource(nitro)
                return nsresponse




from nsbaseresource import NSBaseResource

__author__ = 'ivanxx@gmail.com'

class NSVersion(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSVersion, self).__init__()

                self.options = {'version': '',
                                'mode' : ''}

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

                self.resourcetype = NSVersion.get_resourcetype()

        def get_version(self):
            return self.options['version']

        def get_mode(self):
            return self.options['mode']

        @staticmethod
        def get_resourcetype():
                return "nsversion"

        @staticmethod
        def get(nitro):
            __url = nitro.get_url() + NSVersion.get_resourcetype()
            __json_nsversion = nitro.get(__url).get_response_field(NSVersion.get_resourcetype())
            return NSVersion(__json_nsversion)



from nsbaseresource import NSBaseResource

__author__ = 'ivanxx@gmail.com'

class SystemUser(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(SystemUser, self).__init__()

                self.options = {'username': '',
                                'password' : '',
                                'priority' : ''}

                self.resourcetype = SystemUser.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options:
                                        self.options[key] = json_data[key]

        @staticmethod
        def get_resourcetype():
                return "systemuser"

        def set_username(self, username):
            self.options['username'] = username

        def get_username(self):
            return self.options['username']

        def set_password(self, password):
            self.options['password'] = password

        def get_password(self):
            return self.options['password']

        def set_priority(self, password):
            # The priority, when submitted to the Nitro API, is ignored by the Nitro API
            # and it is always zero
            self.options['priority'] = password

        def get_priority(self):
            # The priority, when retrieve from the Nitro API, is always zero
            return self.options['priority']

        @staticmethod
        def add(nitro, systemuser):
            __systemuser = SystemUser()
            __systemuser.set_username(systemuser.get_username())
            __systemuser.set_password(systemuser.get_password())
            return __systemuser.add_resource(nitro)

        @staticmethod
        def get(nitro, systemuser):
            __systemuser = SystemUser()
            __systemuser.set_username(systemuser.get_username())
            __systemuser.get_resource(nitro, object_name=__systemuser.get_username())
            return __systemuser

        @staticmethod
        def update(nitro, systemuser):
            __systemuser = SystemUser()
            __systemuser.set_username(systemuser.get_username())
            __systemuser.set_password(systemuser.get_password())
            return __systemuser.update_resource(nitro)

        @staticmethod
        def delete(nitro, systemuser):
            __systemuser = SystemUser()
            __systemuser.set_username(systemuser.get_username())
            return __systemuser.delete_resource(nitro)

        @staticmethod
        def get_all(nitro):
            __url = nitro.get_url() + SystemUser.get_resourcetype()
            __json_systemusers = nitro.get(__url).get_response_field(SystemUser.get_resourcetype())

            return [SystemUser(u) for u in __json_systemusers]



from nsbaseresource import NSBaseResource
__author__ = 'Aleksandar Topuzovic'


class NSLBVServerCSVserverBinding(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSLBVServerCSVserverBinding, self).__init__()
                self.options = {'priority': '',
                                'policyname': '',
                                'name': '',
                                'cachevserver': '',
                                'cachetype': '',
                                'hits': ''}

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key]=json_data[key]

                self.resourcetype = NSLBVServerCSVserverBinding.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                """
                Binding object showing the csvserver that can be bound to lbvserver.
                """
                return "lbvserver_csvserver_binding"

        # Read/write properties
        def set_priority(self, priority):
                """
                Priority.
                """
                self.options['priority'] = priority

        def get_priority(self):
                """
                Priority.
                """
                return self.options['priority']

        def set_policyname(self, policyname):
                """
                Name of the policy bound to the LB vserver.
                """
                self.options['policyname'] = policyname

        def get_policyname(self):
                """
                Name of the policy bound to the LB vserver.
                """
                return self.options['policyname']

        def set_name(self, name):
                """
                The virtual server name to which the service is bound.
                Minimum length = 1
                """
                self.options['name'] = name

        def get_name(self):
                """
                The virtual server name to which the service is bound.
                Minimum length = 1
                """
                return self.options['name']

        def set_cachevserver(self, cachevserver):
                """
                Cache virtual server.
                """
                self.options['cachevserver'] = cachevserver

        def get_cachevserver(self):
                """
                Cache virtual server.
                """
                return self.options['cachevserver']

        # Read only properties
        def get_cachetype(self):
                """
                Cache type.
                """
                return self.options['cachetype']

        def get_hits(self):
                """
                Number of hits.
                """
                return self.options['hits']

        @staticmethod
        def get(nitro, lbvservercsvserverbinding):
                """
                Use this API to fetch lb vserver cs vserver binding resource of given name.
                """
                __url = nitro.get_url() + NSLBVServerCSVserverBinding.get_resourcetype() + "/" + lbvservercsvserverbinding.get_name()
                __json_csvservers = nitro.get(__url).get_response_field(NSLBVServerCSVserverBinding.get_resourcetype())
                __csvservers = []
                for json_csvserver in __json_csvservers:
                        __csvservers.append(NSLBVServerCSVserverBinding(json_csvserver))
                return __csvservers



from nsbaseresource import NSBaseResource


class NSHANode(NSBaseResource):

# Configuration for HA node resource.

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSHANode, self).__init__()
                self.options = {'id': '',
                                'ipaddress': '',
                                'inc': '',
                                'hastatus': '',
                                'hasync': '',
                                'haprop': '',
                                'hellointerval': '',
                                'deadinterval': '',
                                'failsafe': '',
                                'name': '',
                                'flags': '',
                                'state': '',
                                'enaifaces': '',
                                'disifaces': '',
                                'hamonifaces': '',
                                'pfifaces': '',
                                'ifaces': '',
                                'network': '',
                                'netmask': '',
                                'ssl2': '',
                                'masterstatetime': '',
                                'routemonitor': ''}

                self.resourcetype = NSHANode.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

        @staticmethod
        def get_resourcetype():
                return "hanode"

        def set_id(self, myid):
                self.options['id'] = myid

        def get_id(self):
                return self.options['id']

        def set_ipaddress(self, ipaddress):
                self.options['ipaddress'] = ipaddress

        def get_ipaddress(self):
                return self.options['ipaddress']

        def set_inc(self, inc):
                self.options['inc'] = inc

        def get_inc(self):
                return self.options['inc']

        def set_hastatus(self, hastatus):
                self.options['hastatus'] = hastatus

        def get_hastatus(self):
                return self.options['hastatus']

        def set_hasync(self, hasync):
                self.options['hasync'] = hasync

        def get_hasync(self):
                return self.options['hasync']

        def set_haprop(self, haprop):
                self.options['haprop'] = haprop

        def get_haprop(self):
                return self.options['haprop']

        def set_hellointerval(self, hellointerval):
                self.options['hellointerval'] = hellointerval

        def get_hellointerval(self):
                return self.options['hellointerval']

        def set_deadinterval(self, deadinterval):
                self.options['deadinterval'] = deadinterval

        def get_deadinterval(self):
                return self.options['deadinterval']

        def set_failsafe(self, failsafe):
                self.options['failsafe'] = failsafe

        def get_failsafe(self):
                return self.options['failsafe']

        def get_name(self):
                return self.options['name']

        def get_flags(self):
                return self.options['flags']

        def get_state(self):
                return self.options['state']

        def get_enaifaces(self):
                return self.options['enaifaces']

        def get_disifaces(self):
                return self.options['disifaces']

        def get_hamonifaces(self):
                return self.options['hamonifaces']

        def get_pfifaces(self):
                return self.options['pfifaces']

        def get_ifaces(self):
                return self.options['ifaces']

        def get_network(self):
                return self.options['network']

        def get_netmask(self):
                return self.options['netmask']

        def get_ssl2(self):
                return self.options['ssl2']

        def get_masterstatetime(self):
                return self.options['masterstatetime']

        def get_routemonitor(self):
                return self.options['routemonitor']

        # Operations methods
        @staticmethod
        def get(nitro, hanode):
                """
                Use this API to fetch hanode resource of given name.
                """
                __hanode = NSHANode()
                __hanode.get_resource(nitro, hanode.get_id())
                return __hanode

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all configured hanode resources.
                """
                __url = nitro.get_url() + NSHANode.get_resourcetype()
                __json_hanodes = nitro.get(__url).get_response_field(NSHANode.get_resourcetype())
                __hanodes = []
                for json_hanode in __json_hanodes:
                        __hanodes.append(NSHANode(json_hanode))
                return __hanodes

        @staticmethod
        def add(nitro, hanode):
                """
                Use this API to add hanode.
                """
                __hanode = NSHANode()
                __hanode.set_id(hanode.get_id())
                __hanode.set_ipaddress(hanode.get_ipaddress())
                __hanode.set_inc(hanode.get_inc())
                return __hanode.add_resource(nitro)

        @staticmethod
        def delete(nitro, hanode):
                """
                Use this API to delete hanode of a given name.
                """
                __hanode = NSHANode()
                nsresponse = __hanode.delete_resource(nitro, hanode.get_id())
                return nsresponse

        @staticmethod
        def update(nitro, hanode):
                """
                Use this API to update a hanode of a given name.
                """
                __hanode = NSHANode()
                __hanode.set_id(hanode.get_id())
                __hanode.set_hastatus(hanode.get_hastatus())
                __hanode.set_hasync(hanode.get_hasync())
                __hanode.set_haprop(hanode.get_haprop())
                __hanode.set_hellointerval(hanode.get_hellointerval())
                __hanode.set_deadinterval(hanode.get_deadinterval())
                __hanode.set_failsafe(hanode.get_failsafe())
                return __hanode.update_resource(nitro)


        # No unset functionality for now.



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSSSLVServerSSLCertKeyBinding(NSBaseResource):

        # General Netscaler configuration object

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSSSLVServerSSLCertKeyBinding, self).__init__()

                self.options = {'certkeyname': '',
                                'cleartextport': '',
                                'crlcheck': '',
                                'ocspcheck': '',
                                'vservername': '',
                                'ca': '',
                                'snicert': ''}

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

                self.resourcetype = NSSSLVServerSSLCertKeyBinding.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "sslvserver_sslcertkey_binding"

        def set_certkeyname(self, certkeyname):
                self.options['certkeyname'] = certkeyname

        def get_certkeyname(self):
                return self.options['certkeyname']

        def set_cleartextport(self, cleartextport):
                self.options['cleartextport'] = cleartextport

        def get_cleartextport(self):
                return self.options['cleartextport']

        def set_crlcheck(self, crlcheck):
                self.options['crlcheck'] = crlcheck

        def get_crlcheck(self):
                return self.options['crlcheck']

        def set_ocspcheck(self, ocspcheck):
                self.options['ocspcheck'] = ocspcheck

        def get_ocspcheck(self):
                return self.options['ocspcheck']

        def set_vservername(self, vservername):
                self.options['vservername'] = vservername

        def get_vservername(self):
                return self.options['vservername']

        def set_ca(self, ca):
                self.options['ca'] = ca

        def get_ca(self):
                return self.options['ca']

        def set_snicert(self, snicert):
                self.options['snicert'] = snicert

        def get_snicert(self):
                return self.options['snicert']

        @staticmethod
        def get(nitro, resource):
                """
                Use this API to fetch SSL certkey resource of given name.
                """
                __url = nitro.get_url() + NSSSLVServerSSLCertKeyBinding.get_resourcetype() + "/" + resource.get_vservername()
                __json_resources = nitro.get(__url).get_response_field(NSSSLVServerSSLCertKeyBinding.get_resourcetype())
                __resources = []
                for json_resource in __json_resources:
                        __resources.append(NSSSLVServerSSLCertKeyBinding(json_resource))
                return __resources

        @staticmethod
        def add(nitro, resource):
                """
                Use this API to add resource.
                """
                __resource = NSSSLVServerSSLCertKeyBinding()
                __resource.set_vservername(resource.get_vservername())
                __resource.set_certkeyname(resource.get_certkeyname())
                __resource.set_ca(resource.get_ca())
                __resource.set_crlcheck(resource.get_crlcheck())
                __resource.set_snicert(resource.get_snicert())
                __resource.set_ocspcheck(resource.get_ocspcheck())
                return __resource.add_resource(nitro)

        @staticmethod
        def delete(nitro, resource):
                """
                Use this API to delete server of a given name.
                """
                __resource = NSSSLVServerSSLCertKeyBinding()
                __resource.set_vservername(resource.get_vservername())
                __resource.set_certkeyname(resource.get_certkeyname())
                __resource.set_ca(resource.get_ca())
                __resource.set_crlcheck(resource.get_crlcheck())
                __resource.set_snicert(resource.get_snicert())
                nsresponse = __resource.delete_resource(nitro)
                return nsresponse



from nsbaseresource import NSBaseResource
__author__ = 'timl'


class NSResponderPolicyLabelBinding(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSResponderPolicyLabelBinding, self).__init__()
                self.options = {
                        'labelname' : '',
                        'policyname' : '',
                        'priority' : '',
                        'gotopriorityexpression' : '',
                        'flags' : '',
                        'labeltype' : '',
                        'invoke_labelname' : '',
                }

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key]=json_data[key]

                self.resourcetype = NSResponderPolicyLabelBinding.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "responderpolicylabel_responderpolicy_binding"

        def set_labelname(self, labelname):
                """
                Responder policy label.

                Default value: 0
                """
                self.options['labelname'] = labelname

        def get_labelname(self):
                """
                Responder policy label.

                Default value: 0
                """
                return self.options['labelname']

        def set_policyname(self, policyname):
                """
                Bound responder policies to this responder policy label.

                Default value: 0
                """
                self.options['policyname'] = policyname

        def get_policyname(self):
                """
                Bound responder policies to this responder policy label.

                Default value: 0
                """
                return self.options['policyname']

        def set_priority(self, priority):
                """
                Priority for the policy.

                Default value: 0
                """
                self.options['priority'] = priority

        def get_priority(self):
                """
                Priority for the policy.

                Default value: 0
                """
                return self.options['priority']

        def set_gotopriorityexpression(self, gotopriorityexpression):
                """
                Expression specifying the priority of the next policy which will get evaluated if the current policy rule
                evaluates to TRUE.

                Default value: 0
                """
                self.options['gotopriorityexpression'] = gotopriorityexpression

        def get_gotopriorityexpression(self):
                """
                Expression specifying the priority of the next policy which will get evaluated if the current policy rule
                evaluates to TRUE.

                Default value: 0
                """
                return self.options['gotopriorityexpression']

        def set_flags(self, flags):
                """
                Bind flags.

                Default value: 0
                """
                self.options['flags'] = flags

        def get_flags(self):
                """
                Bind flags.

                Default value: 0
                """
                return self.options['flags']

        def set_labeltype(self, labeltype):
                """
                The invocation type.

                Default value: 0
                """
                self.options['labeltype'] = labeltype

        def get_labeltype(self):
                """
                The invocation type.

                Default value: 0
                """
                return self.options['labeltype']

        def set_invoke_labelname(self, invoke_labelname):
                """
                Name of the label invoked.

                Default value: 0
                """
                self.options['invoke_labelname'] = invoke_labelname

        def get_invoke_labelname(self):
                """
                Name of the label invoked.

                Default value: 0
                """
                return self.options['invoke_labelname']

        # Operations methods
        @staticmethod
        def get(nitro, responderpolicylabel_binding):
                """
                Use this API to fetch all configured responderpolicylabel_binding resources.
                """
                __url = nitro.get_url() + NSResponderPolicyLabelBinding.get_resourcetype() + "/" + responderpolicylabel_binding.get_labelname()
                __json_resources = nitro.get(__url).get_response_field(NSResponderPolicyLabelBinding.get_resourcetype())
                __resources = []
                for json_resource in __json_resources:
                        __resources.append(NSResponderPolicyLabelBinding(json_resource))
                return __resources


        @staticmethod
        def add(nitro, responderpolicylabel_binding):
                """
                Use this API to add responderpolicylabel_binding.
                """
                __responderpolicylabel_binding = NSResponderPolicyLabelBinding()
                __responderpolicylabel_binding.set_labelname(responderpolicylabel_binding.get_labelname())
                __responderpolicylabel_binding.set_policyname(responderpolicylabel_binding.get_policyname())
                __responderpolicylabel_binding.set_priority(responderpolicylabel_binding.get_priority())
                __responderpolicylabel_binding.set_gotopriorityexpression(responderpolicylabel_binding.get_gotopriorityexpression())
                __responderpolicylabel_binding.set_flags(responderpolicylabel_binding.get_flags())
                __responderpolicylabel_binding.set_labeltype(responderpolicylabel_binding.get_labeltype())
                __responderpolicylabel_binding.set_invoke_labelname(responderpolicylabel_binding.get_invoke_labelname())
                return __responderpolicylabel_binding.update_resource(nitro)

        @staticmethod
        def delete(nitro, responderpolicylabel_binding):
                """
                Use this API to delete responderpolicylabel_binding of a given name.
                """
                __responderpolicylabel_binding = NSResponderPolicyLabelBinding()
                __name = responderpolicylabel_binding.get_labelname()
                __responderpolicylabel_binding.set_labelname(__name)
                __responderpolicylabel_binding.set_policyname(responderpolicylabel_binding.get_policyname())
                nsresponse = __responderpolicylabel_binding.delete_resource(nitro, __name)
                return nsresponse



#!/usr/bin/env python
#
# Netscaler NITRO controller
#
#import the necessary libraries
import argparse
import sys
import nsnitro
from nsnitro.nsnitro import *
from nsnitro.nsresources.nsconfig import NSConfig
from nsnitro.nsutil import *
from nsnitro.nsresources.nslbvserver import NSLBVServer
from nsnitro.nsresources.nsservice import NSService
from nsnitro.nsresources.nsserver import NSServer
from nsnitro.nsresources.nscsvserver import NSCSVServer
from nsnitro.nsresources.nsacl import NSAcl
from nsnitro.nsresources.nsacls import NSAcls
from nsnitro.nsresources.nshanode import NSHANode
from nsnitro.nsresources.nsservicegroupserverbinding import NSServiceGroupServerBinding

if __name__ == "__main__":
        parser = argparse.ArgumentParser(description='Netscaler NITRO controller')
        parser.add_argument('--lbip', metavar='IP', required=True, help='lb ip address')
        parser.add_argument('--user', metavar='USERNAME', default='api_user', help='lb username')
        parser.add_argument('--password', metavar='PASSWORD', default='api_user', help='lb password')

        parser.add_argument('--addlbvserver', metavar='LBVSERVERNAME', help='enable lb vserver')
        parser.add_argument('--enablelbvserver', metavar='LBVSERVERNAME', help='enable lb vserver')
        parser.add_argument('--disablelbvserver', metavar='LBVSERVERNAME', help='disable lb vserver')
        parser.add_argument('--renamelbvserver', metavar=('LBVSERVERNAME', 'NEWNAME'), nargs=2, help='rename lb vserver from NAME to NEWNAME')

        parser.add_argument('--getlbvserver', metavar='LBVSERVERNAME', help='show lb vserver')
        parser.add_argument('--getlbvserverslist', action='store_true', help='show lb vservers list')
        parser.add_argument('--getlbvserverstatus', metavar='LBVSERVERNAME', help='show lb vserver status')
        parser.add_argument('--getlbvserverstatusfull', metavar='LBVSERVERNAME', help='show lb vserver status with bound services')
        parser.add_argument('--getlbvserversstatus', action='store_true', help='show lb vservers status')

        parser.add_argument('--enablecsvserver', metavar='CSVSERVERNAME', help='enable cs vserver')
        parser.add_argument('--disablecsvserver', metavar='CSVSERVERNAME', help='disable cs vserver')
        parser.add_argument('--renamecsvserver', metavar=('CSVSERVERNAME', 'NEWNAME'), nargs=2, help='rename cs vserver from NAME to NEWNAME')
        parser.add_argument('--getcsvserver', metavar='CSVSERVERNAME', help='show cs vserver')
        parser.add_argument('--getcsvserverslist', action='store_true', help='show cs vservers list')
        parser.add_argument('--getcsvserverstatus', metavar='CSVSERVERNAME', help='show cs vserver status')
        parser.add_argument('--getcsvserversstatus', action='store_true', help='show cs vservers status')

        parser.add_argument('--addservice', metavar=('SERVICENAME', 'LBVSERVERNAME'), nargs=2, help='add service SERVICENAME to lb vserver LBVSERVERNAME (ip and port also requires)')
        parser.add_argument('--enableservice', metavar='SERVICENAME', help='enable service')
        parser.add_argument('--disableservice', metavar='SERVICENAME', help='disable service')
        parser.add_argument('--renameservice', metavar=('NAME', 'NEWNAME'), nargs=2, help='rename service from NAME to NEWNAME')

        parser.add_argument('--getservice', metavar='SERVICENAME', help='show service')
        parser.add_argument('--getservicestatus', metavar='SERVICENAME', help='show service status')
        parser.add_argument('--getserviceslist', action='store_true', help='show services list')
        parser.add_argument('--getservicesstatus', action='store_true', help='show services status')
        parser.add_argument('--bindservice', metavar=('SERVICENAME', 'LBVSERVERNAME'), nargs=2, help='bind service SERVICENAME to lb vserver LBVSERVERNAME ')

        parser.add_argument('--getserver', metavar='SERVERNAME', help='show server')
        parser.add_argument('--getserverslist', action='store_true', help='show servers list')
        parser.add_argument('--getserversstatus', action='store_true', help='show servers status')
        parser.add_argument('--enableserver', metavar='SERVERNAME', help='enable server')
        parser.add_argument('--disableserver', metavar='SERVERNAME', help='disable server')
        parser.add_argument('--renameserver', metavar=('NAME', 'NEWNAME'), nargs=2, help='rename server from NAME to NEWNAME')

        parser.add_argument('--saveconfig', action='store_true', help='save loadbalancer config')

        parser.add_argument('--getprimarylb', action='store_true', help='If you netscaler is in a cluster show the node that is primary')

        parser.add_argument('--dargs', action='store_true', help='show service')
        # additional arguments
        parser.add_argument('--port', metavar='PORT', help='port number')
        parser.add_argument('--ip', metavar='IP', help='IP address')
        parser.add_argument('--servicetype', metavar='SERVICETYPE', choices=['TCP','HTTP'], default='TCP', help='Service type')
        parser.add_argument('--clttimeout', metavar='CLTTIMEOUT', default=9000, help='Clt timeout')
        parser.add_argument('--svrtimeout', metavar='SRVTIMEOUT', default=9000, help='service timeout')
        parser.add_argument('--persistencetype', metavar='PERSISTENCETYPE', default='NONE', help='persistence type')
        parser.add_argument('--bindingweight', metavar='BINDINGWIIGHT', default=40, help='weight parameter for binding service')

        # ACL related arguments
        parser.add_argument('--getacl', metavar='ACLNAME', help='show acl')
        parser.add_argument('--getaclslist', action='store_true', help='show acl list')
        parser.add_argument('--addacl', metavar='ACLNAME', help='add acl')
        parser.add_argument('--delacl', metavar='ACLNAME', help='del acl')
        parser.add_argument('--applyacls', action='store_true', help='apply acls to kernel')
        parser.add_argument('--clearacls', action='store_true', help='clear acls')
        parser.add_argument('--renumberacls', action='store_true', help='renumber acls')

        parser.add_argument('--bindservertoservicegroup', action='store_true', help='bind a server to a service group')
        parser.add_argument('--unbindserverfromservicegroup', action='store_true', help='bind a server to a service group')
        parser.add_argument('--servername', action='store', help='server object to manipulate')
        parser.add_argument('--servicegroupname', action='store', help='service group object to manipulate')
        parser.add_argument('--serviceport', action='store', help='service port')

        args = parser.parse_args()

        if args.dargs:
                print(args)
                sys.exit(0)

        nitro = NSNitro(args.lbip, args.user, args.password)

        try:
                nitro.login()

                if args.bindservertoservicegroup:
                  if not args.servername or not args.servicegroupname or not args.serviceport: 
                    print "--servername, --servicename, and --serviceport are required for binding a server to a service group"
                    nitro.logout()
                    sys.exit(0)
                  else:
                    svcgrpbinding=NSServiceGroupServerBinding()
                    svcgrpbinding.set_servername(args.servername)
                    svcgrpbinding.set_servicegroupname(args.servicegroupname)
                    svcgrpbinding.set_port(args.serviceport)
                    try:
                      NSServiceGroupServerBinding.add(nitro, svcgrpbinding)
                      print "bound server %s to service group %s on port %s" % (args.servername, args.servicegroupname, args.serviceport)
                    except nsnitro.nsexceptions.nsexceptions.NSNitroNserrExist as e:
                      print "Error: ", e.message
                    finally:
                      nitro.logout()
                      sys.exit(0)
                    
                if args.unbindserverfromservicegroup:
                  if not args.servername or not args.servicegroupname or not args.serviceport: 
                    print "--servername, --servicename, and --serviceport are required for binding a server to a service group"
                    nitro.logout()
                    sys.exit(0)
                  else:
                    svcgrpbinding=NSServiceGroupServerBinding()
                    svcgrpbinding.set_servername(args.servername)
                    svcgrpbinding.set_servicegroupname(args.servicegroupname)
                    svcgrpbinding.set_port(args.serviceport)
                    try:
                      NSServiceGroupServerBinding.delete(nitro, svcgrpbinding)
                      print "unbound server %s from service group %s on port %s" % (args.servername, args.servicegroupname, args.serviceport)
                    except nsnitro.nsexceptions.nsexceptions.NSNitroNserrNoent as e:
                      print "Error: ", e.message
                    finally:
                      nitro.logout()
                      sys.exit(0)
                    

                if args.addlbvserver:
                        if not args.port or not args.ip:
                          print "--ip and --port are required for adding LB Vserver"
                          nitro.logout()
                          sys.exit(0)
                        lbvserver = NSLBVServer()
                        lbvserver.set_name(args.addlbvserver)
                        lbvserver.set_ipv46(args.ip)
                        lbvserver.set_port(args.port)
                        lbvserver.set_clttimeout(args.clttimeout)
                        lbvserver.set_persistencetype(args.persistencetype)
                        lbvserver.set_servicetype(args.servicetype)
                        NSLBVServer.add(nitro, lbvserver)
                        print "lb vserver %s (%s:%d/%s) was added" % (args.addlbvserver, args.ip, args.port, args.servicetype)
                        nitro.logout()
                        sys.exit(0)

                if args.enablelbvserver:
                        lbvserver = NSLBVServer()
                        lbvserver.set_name(args.enablelbvserver)
                        NSLBVServer.enable(nitro, lbvserver)
                        print "Enabled lb vserver: %s" % args.enablelbvserver
                        nitro.logout()
                        sys.exit(0)

                if args.saveconfig:
                        NSConfig.save(nitro)
                        print "Saved Netscaler configuration"
                        nitro.logout()
                        sys.exit(0)

                if args.getprimarylb:
                        ha_node = NSHANode()
                        for node in ha_node.get_all(nitro):
                            state = node.get_state().lower()
                            if state == 'primary':
                                print(node.get_ipaddress())
                                break
                        nitro.logout()
                        sys.exit(0)

                if args.disablelbvserver:
                        lbvserver = NSLBVServer()
                        lbvserver.set_name(args.disablelbvserver)
                        NSLBVServer.disable(nitro, lbvserver)
                        print "Disabled lb vserver: %s" % args.disablelbvserver
                        nitro.logout()
                        sys.exit(0)

                if args.getlbvserver:
                        lbvserver = NSLBVServer()
                        lbvserver.set_name(args.getlbvserver)
                        lbvserver = NSLBVServer.get(nitro, lbvserver)
                        print "--- LB vserver: " + lbvserver.get_name() + " ---"
                        for k in sorted(lbvserver.options.iterkeys(), key=lambda k: k):
                                print "\t%s: %s" % (k, lbvserver.options[k])

                        nitro.logout()
                        sys.exit(0)

                if args.enablecsvserver:
                        csvserver = NSCSVServer()
                        csvserver.set_name(args.enablecsvserver)
                        NSCSVServer.enable(nitro, csvserver)
                        print "Enabled cs vserver: %s" % args.enablecsvserver
                        nitro.logout()
                        sys.exit(0)

                if args.disablecsvserver:
                        csvserver = NSCSVServer()
                        csvserver.set_name(args.disablecsvserver)
                        NSCSVServer.disable(nitro, csvserver)
                        print "Disabled cs vserver: %s" % args.disablecsvserver
                        nitro.logout()
                        sys.exit(0)

                if args.getcsvserver:
                        csvserver = NSCSVServer()
                        csvserver.set_name(args.getcsvserver)
                        csvserver = NSCSVServer.get(nitro, csvserver)
                        print "--- CS vserver: " + csvserver.get_name() + " ---"
                        for k in sorted(csvserver.options.iterkeys(), key=lambda k: k):
                                print "\t%s: %s" % (k, csvserver.options[k])

                        nitro.logout()
                        sys.exit(0)

                if args.addservice:
                        if not args.port:
                          print "--port is required for adding service"
                          nitro.logout()
                          sys.exit(0)
                        service = NSService()
                        service.set_port(args.port)
                        service.set_servicetype(args.servicetype)
                        service.set_clttimeout(args.clttimeout)
                        service.set_svrtimeout(args.svrtimeout)
                        service.set_name(args.addservice[0])
                        service.set_servername(args.addservice[1])
                        NSService.add(nitro, addservice)
                        print "Service '%s:%d/%s' was added to '%s'." % (args.addservice[0], args.port, args.servicetype, args.addservice[1])
                        nitro.logout()
                        sys.exit(0)

                if args.bindservice:
                        binding = NSLBVServerServiceBinding()
                        binding.set_weight(args.bindingweight)
                        binding.set_servicename(args.bindservice[0])
                        binding.set_name(args.bindservice[1])
                        NSLBVServerServiceBinding.add(nitro, binding)
                        print "Service '%s' was binded to LB vserver '%s' with weight %d." % (args.bindservice[0], args.bindservice[1], args.bindingweight)
                        nitro.logout()
                        sys.exit(0)

                if args.enableservice:
                        service = NSService()
                        service.set_name(args.enableservice)
                        NSService.enable(nitro, service)
                        print "Enabled service: %s" % args.enableservice
                        nitro.logout()
                        sys.exit(0)

                if args.disableservice:
                        service = NSService()
                        service.set_name(args.disableservice)
                        NSService.disable(nitro, service)
                        print "Disabled service: %s" % args.disableservice
                        nitro.logout()
                        sys.exit(0)

                if args.getservice:
                        service = NSService()
                        service.set_name(args.getservice)
                        service = NSService.get(nitro, service)

                        print "--- Service: " + service.get_name() + " ---"
                        for k in sorted(service.options.iterkeys(), key=lambda k: k):
                                print "\t%s: %s" % (k, service.options[k])

                        nitro.logout()
                        sys.exit(0)

                if args.getserviceslist:
                        services = NSService().get_all(nitro)
                        print "-- Configured services ---"
                        for service in sorted(services, key=lambda k: k.get_name()):
                                print "\t" + service.get_name()
                        nitro.logout()
                        sys.exit(0)

                if args.getlbvserverslist:
                        vservers = NSLBVServer().get_all(nitro)
                        print "-- Configured LB vservers ---"
                        for vserver in sorted(vservers, key=lambda k: k.get_name()):
                                print "\t" + vserver.get_name()
                        nitro.logout()
                        sys.exit(0)

                if args.getcsvserverslist:
                        vservers = NSCSVServer().get_all(nitro)
                        print "-- Configured CS vservers ---"
                        for vserver in sorted(vservers, key=lambda k: k.get_name()):
                                print "\t" + vserver.get_name()
                        nitro.logout()
                        sys.exit(0)

                if args.getservicesstatus:
                        services = NSService().get_all(nitro)
                        print "-- Configured services (with status) ---"
                        for service in sorted(services, key=lambda k: k.get_name()):
                                print "\t" + service.get_name() + ": " + service.get_svrstate()
                        nitro.logout()
                        sys.exit(0)

                if args.getlbvserversstatus:
                        vservers = NSLBVServer().get_all(nitro)
                        print "-- Configured LB vservers (with status) ---"
                        for vserver in sorted(vservers, key=lambda k: k.get_name()):
                                print vserver.get_name() + ": " + vserver.get_effectivestate()
                        nitro.logout()
                        sys.exit(0)

                if args.getcsvserversstatus:
                        vservers = NSCSVServer().get_all(nitro)
                        print "-- Configured CS vservers (with status) ---"
                        for vserver in sorted(vservers, key=lambda k: k.get_name()):
                                print vserver.get_name() + ": " + vserver.get_curstate()
                        nitro.logout()
                        sys.exit(0)

                if args.getlbvserverstatus:
                        vserver = NSLBVServer()
                        vserver.set_name(args.getlbvserverstatus)
                        vserver = NSLBVServer().get(nitro, vserver)
                        print vserver.get_name() + ": " + vserver.get_effectivestate()
                        nitro.logout()
                        sys.exit(0)

                if args.getlbvserverstatusfull:
                        binding = NSLBVServerServiceBinding()
                        binding.set_name(args.getlbvserverstatusfull)
                        binded_services = NSLBVServerServiceBinding().get(nitro, binding)
                        print "-- | VServerName | Servicename | State | Type | IP | Port | Weight | ---"
                        for s in binded_services:
                            print ' | ', s.get_name(),        ' | ', s.get_servicename(), ' | ', s.get_curstate(),
                            print ' | ', s.get_servicetype(), ' | ', s.get_ipv46(),       ' | ', s.get_port(),
                            print ' | ', s.get_weight()
                        nitro.logout()
                        sys.exit(0)

                if args.getcsvserverstatus:
                        vserver = NSCSVServer()
                        vserver.set_name(args.getcsvserverstatus)
                        vserver = NSCSVServer().get(nitro, vserver)
                        print vserver.get_name() + ": " + vserver.get_curstate()
                        nitro.logout()
                        sys.exit(0)

                if args.getservicestatus:
                        service = NSService()
                        service.set_name(args.getservicestatus)
                        service = NSService.get(nitro, service)
                        print service.get_name() + ": " + service.get_svrstate()
                        nitro.logout()
                        sys.exit(0)


                if args.enableserver:
                        server = NSServer()
                        server.set_name(args.enableserver)
                        NSServer.enable(nitro, server)
                        print "Enabled server: %s" % args.enableserver
                        nitro.logout()
                        sys.exit(0)

                if args.disableserver:
                        server = NSServer()
                        server.set_name(args.disableserver)
                        NSServer.disable(nitro, server)
                        print "Disabled server: %s" % args.disableserver
                        nitro.logout()
                        sys.exit(0)

                if args.getserver:
                        server = NSServer()
                        server.set_name(args.getserver)
                        server = NSServer.get(nitro, server)
                        print "--- Server: " + server.get_name() + " ---"
                        for k in sorted(server.options.iterkeys(), key=lambda k: k):
                                print "\t%s: %s" % (k, server.options[k])
                        nitro.logout()
                        sys.exit(0)


                if args.getserverslist:
                        servers = NSServer().get_all(nitro)
                        print "-- Configured servers ---"
                        for server in sorted(servers, key=lambda k: k.get_name()):
                                print "\t" + server.get_name()
                        nitro.logout()
                        sys.exit(0)

                if args.getserversstatus:
                        servers = NSServer().get_all(nitro)
                        print "-- Configured servers (with status) ---"
                        for server in sorted(servers, key=lambda k: k.get_name()):
                                print "\t" + server.get_name() + ": " + server.get_state()
                        nitro.logout()
                        sys.exit(0)

                if args.renameservice:
                        service = NSService()
                        service.set_name(args.renameservice[0])
                        service.set_newname(args.renameservice[1])
                        NSService.rename(nitro, service)
                        print "Renamed service from '%s' to '%s'." % (args.renameservice[0], args.renameservice[1])
                        nitro.logout()
                        sys.exit(0)

                if args.renamelbvserver:
                        lbvserver = NSLBVServer()
                        lbvserver.set_name(args.renamelbvserver[0])
                        lbvserver.set_newname(args.renamelbvserver[1])
                        NSLBVServer.rename(nitro, lbvserver)
                        print "Renamed LB vserver from '%s' to '%s'." % (args.renamelbvserver[0], args.renamelbvserver[1])
                        nitro.logout()
                        sys.exit(0)

                if args.renamecsvserver:
                        csvserver = NSCSVServer()
                        csvserver.set_name(args.renamecsvserver[0])
                        csvserver.set_newname(args.renamecsvserver[1])
                        NSCSVServer.rename(nitro, csvserver)
                        print "Renamed CS vserver from '%s' to '%s'." % (args.renamecsvserver[0], args.renamecsvserver[1])
                        nitro.logout()
                        sys.exit(0)

                if args.renameserver:
                        server = NSServer()
                        server.set_name(args.renameserver[0])
                        server.set_newname(args.renameserver[1])
                        NSServer.rename(nitro, server)
                        print "Renamed server from '%s' to '%s'." % (args.renameserver[0], args.renameserver[1])
                        nitro.logout()
                        sys.exit(0)

                if args.getacl:
                        acl = NSAcl()
                        acl.set_aclname(args.getacl)
                        acl = NSAcl.get(nitro, acl)

                        print "--- ACL: " + acl.get_aclname() + " ---"
                        for k in sorted(acl.options.iterkeys(), key=lambda k: k):
                                print "\t%s: %s" % (k, acl.options[k])

                        nitro.logout()
                        sys.exit(0)

                if args.getaclslist:
                        acls = NSAcl().get_all(nitro)
                        print "-- Configured ACLs ---"
                        for acl in sorted(acls, key=lambda k: k.get_aclname()):
                                print "\t" + acl.get_aclname()
                        nitro.logout()
                        sys.exit(0)

                if args.applyacls:
                        NSAcls.apply(nitro)
                        print "Applied all ACLs to netscaler kernel."
                        nitro.logout()
                        sys.exit(0)

                if args.clearacls:
                        NSAcls.clear(nitro)
                        print "Cleared ACLs on the netscaler."
                        nitro.logout()
                        sys.exit(0)

                if args.renumberacls:
                        NSAcls.renumber(nitro)
                        print "Renumbered ACLs on the netscaler."
                        nitro.logout()
                        sys.exit(0)

                if args.delacl:
                        acl = NSAcl()
                        acl.set_aclname(args.delacl)
                        acl = NSAcl.delete(nitro, acl)
                        print "ACL '%s' was deleted. Do not forget to run --applyacls to activate it." % (args.delacl)
                        nitro.logout()
                        sys.exit(0)

                print "No action specified. Exiting."
                sys.exit(0)
        except NSNitroError, e:
                print "Error: %s" % e.message



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'

class NSVLANNSIPBinding(NSBaseResource):

        # General Netscaler configuration object

        def __init__(self, json_data = None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSVLANNSIPBinding, self).__init__()

                self.options = {
                        'ipaddress' : '',
                        'netmask' : '',
                        'rnat' : '',
                        'id' : '',
                }

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key]=json_data[key]

                self.resourcetype = NSVLANNSIPBinding.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "vlan_nsip_binding"

        def set_ipaddress(self, ipaddress):
                self.options['ipaddress'] = ipaddress

        def get_ipaddress(self):
                return self.options['ipaddress']

        def set_netmask(self, netmask):
                self.options['netmask'] = netmask

        def get_netmask(self):
                return self.options['netmask']

        def set_rnat(self, rnat):
                self.options['rnat'] = rnat

        def get_rnat(self):
                return self.options['rnat']

        def set_id(self, id):
                self.options['id'] = id

        def get_id(self):
                return self.options['id']


        @staticmethod
        def add(nitro, resource):
                __resource = NSVLANNSIPBinding()
                __resource.set_id(resource.get_id())
                __resource.set_ipaddress(resource.get_ipaddress())
                __resource.set_netmask(resource.get_netmask())
                return __resource.add_resource(nitro)

        @staticmethod
        def delete(nitro, resource):
                __resource = NSVLANNSIPBinding()
                __resource.set_id(resource.get_id())
                __resource.set_ipaddress(resource.get_ipaddress())
                __resource.set_netmask(resource.get_netmask())
                nsresponse = __resource.delete_resource(nitro, object_name=__resource.get_id())
                return nsresponse

        @staticmethod
        def get(nitro, resource):
                """
                Use this API to fetch service resource of given name.
                """
                __resource = NSVLANNSIPBinding()
                __resource.set_id(resource.get_id())
                __resource.get_resource(nitro, object_name=resource.get_id())
                return __resource



from nsbaseresource import NSBaseResource

class NSAcl(NSBaseResource):

# Configuration for ACL entry resource.

        def __init__(self,json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSAcl, self).__init__()
                self.options={
                        'aclname' : '',
                        'aclaction' : '',
                        'srcip' : '',
                        'srcipop' : '',
                        'srcipval' : '',
                        'srcport' : '',
                        'srcportop' : '',
                        'srcportval' : '',
                        'destip' : '',
                        'destipop' : '',
                        'destipval' : '',
                        'destport' : '',
                        'destportop' : '',
                        'destportval' : '',
                        'ttl' : '',
                        'srcmac' : '',
                        'protocol' : '',
                        'protocolnumber' : '',
                        'vlan' : '',
                        'Interface' : '',
                        'established' : '',
                        'icmptype' : '',
                        'icmpcode' : '',
                        'priority' : '',
                        'state' : '',
                        'logstate' : '',
                        'ratelimit' : '',
                        'hits': '',
                        'kernelstate': '',
                        '__count': ''
                } 

                self.resourcetype = NSAcl.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key]=json_data[key]

        @staticmethod
        def get_resourcetype():
            return "nsacl"

        def set_aclname(self, aclname):
	    """
            The alphanumeric name of the ACL.
            Default value: 0
            Minimum length =  1.
            """
            self.options['aclname'] = aclname

        def get_aclname(self):
            """
            The action associated with the ACL.
            Default value: 0
            """
            return self.options['aclname']

        def set_aclaction(self, aclaction):
            """
            The action associated with the ACL.
            Default value: 0
            """
            self.options['aclaction'] = aclaction

        def get_aclaction(self):
            """
            The action associated with the ACL.
            Default value: 0
            """
            return self.options['aclaction']

        def set_srcip(self, srcip):
            """
            The source IP address (range).
            Default value: 0
            """
            self.options['srcip'] = srcip

        def get_srcip(self):
            """
            The source IP address (range).
            Default value: 0
            """
            return self.options['srcip']

        def set_srcipop(self, srcipop):
            """ 
            Logical operator.
            Default value: 0
            """ 
            self.options['srcipop'] = srcipop

        def get_srcipop(self):
            """
            Logical operator.
            Default value: 0
            """
            return self.options['srcipop']

        def set_srcipval(self, srcipval):
            """
            The source IP address (range).
            Default value: 0
            """
            self.options['srcipval'] = srcipval

        def get_srcipval(self):
            """
            The source IP address (range).
            Default value: 0
            """
            return self.options['srcipval']

        def set_srcport(self, srcport):
            """
            The source Port (range).
            Default value: 0
            """
            self.options['srcport'] = srcport

        def get_srcport(self):
            """
            The source Port (range).
            Default value: 0
            """
            return self.options['srcport']

        def set_srcportop(self, srcportop):
            """
            Logical operator.
            Default value: 0
            """ 
            self.options['srcportop'] = srcportop

        def get_srcportop(self):
            """
            Logical operator.
            Default value: 0
            """ 
            return self.options['srcportop']

        def set_srcportval(self, srcportval):
            """
            The source Port (range).
            Default value: 0
            Maximum length =  65535.
            """
            self.options['srcportval'] = srcportval

        def get_srcportval(self):
            """
            The source Port (range).
            Default value: 0
            Maximum length =  65535.
            """
            return self.options['srcportval']

        def set_destip(self, destip):
            """
            The destination IP address (range).
            Default value: 0
            """
            self.options['destip']= destip

        def get_destip(self):
            """
            The destination IP address (range).
            Default value: 0
            """
            return self.options['destip']

        def set_destipop(self, destipop):
            """
            Logical operator.
            Default value: 0
            """
            self.options['destipop'] = destipop

        def get_destipop(self):
            """
            Logical operator.
            Default value: 0
            """
            return self.options['destipop']

        def set_destipval(self, destipval):
            """
            The destination IP address (range).
            Default value: 0
            """
            self.options['destipval'] = destipval

        def get_destipval(self):
            """
            The destination IP address (range).
            Default value: 0
            """
            return self.options['destipval']

        def set_destport(self, destport):
            """
            The destination Port (range).
            Default value: 0
            """
            self.options['destport'] = destport

        def get_destport(self):
            """
            The destination Port (range).
            Default value: 0
            """
            return self.options['destport']

        def set_destportop(self, destportop):
            """
            Logical operator.
            Default value: 0
            """
            self.options['destportop'] = destportop

        def get_destportop(self):
            """
            Logical operator.
            Default value: 0
            """
            return self.options['destportop']

        def set_destportval(self, destportval):
            """
            The destination Port (range).
            Default value: 0
            Maximum length =  65535.
            """
            self.options['destportval'] = destportval

        def get_destportval(self):
            """
            The destination Port (range).
            Default value: 0
            Maximum length =  65535.
            """
            return self.options['destportval']

        def set_ttl(self, ttl):
            """
            The time to expire this ACL(in seconds).
            Default value: 0
            Minimum value =  1
            Maximum value =  0x7FFFFFFF
            """
            self.options['ttl'] = ttl

        def get_ttl(self):
            """
            The time to expire this ACL(in seconds).
            Default value: 0
            Minimum value =  1
            Maximum value =  0x7FFFFFFF
            """
            return self.options['ttl']

        def set_srcmac(self, srcmac):
            """
            The source MAC address.
            Default value: 0
            """
            self.options['srcmac'] = srcmac

        def get_srcmac(self):
            """
            The source MAC address.
            Default value: 0
            """
            return self.options['srcmac']

        def set_protocol(self, protocol):
            """
            The IP protocol name.
            Default value: 0
            """
            self.options['protocol'] = protocol

        def get_protocol(self):
            """
            The IP protocol name.
            Default value: 0
            """
            return self.options['protocol']

        def set_protocolnumber(self, protocolnumber):
            """
            The IP protocol number (decimal).
            Default value: 0
            Minimum value =  1
            Maximum value =  255
            """
            self.options['protocolnumber'] = protocolnumber

        def get_protocolnumber(self):
            """
            The IP protocol number (decimal).
            Default value: 0
            Minimum value =  1
            Maximum value =  255
            """
            return self.options['protocolnumber']

        def set_vlan(self, vlan):
            """
            The VLAN number.
            Default value: 0
            Minimum value =  1
            Maximum value =  4094
            """
            self.options['vlan'] = vlan

        def get_vlan(self):
            """
            The VLAN number.
            Default value: 0
            Minimum value =  1
            Maximum value =  4094
            """
            return self.options['vlan']

        def set_Interface(self, Interface):
            """
            The physical interface.
            Default value: 0
            """
            self.options['Interface'] = Interface

        def get_Interface(self):
            """
            The physical interface.
            Default value: 0
            """
            return self.options['Interface']

        def set_established(self, established):
            """
            This argument indicates that the ACL should be used for TCP response traffic only.
            """
            self.options['established'] = established

        def get_established(self):
            """
            This argument indicates that the ACL should be used for TCP response traffic only.
            """
            return self.options['established']

        def set_icmptype(self, icmptype):
            """
            The ICMP message type.
            Default value: 65536
            Minimum value =  0
            Maximum value =  255
            """
            self.options['icmptype'] = icmptype

        def get_icmptype(self):
            """
            The ICMP message type.
            Default value: 65536
            Minimum value =  0
            Maximum value =  255
            """
            return self.options['icmptype']

        def set_icmpcode(self, icmpcode):
            """
            The ICMP message code.
            Default value: 65536
            Minimum value =  0
            Maximum value =  255
            """
            self.options['icmpcode'] = icmpcode

        def get_icmpcode(self):
            """
            The ICMP message code.
            Default value: 65536
            Minimum value =  0
            Maximum value =  255
            """
            return self.options['icmpcode']

        def set_priority(self, priority):
            """
            The priority of the ACL.
            Default value: 0
            Minimum value =  1
            Maximum value =  10240
            """
            self.options['priority'] = priority

        def get_priority(self):
            """
            The priority of the ACL.
            Default value: 0
            Minimum value =  1
            Maximum value =  10240
            """
            return self.options['priority']

        def set_state(self, state):
            """
            The state of the ACL.
            Default value: ENABLED
            """
            self.options['state'] = state

        def get_state(self):
            """
            The state of the ACL.
            Default value: ENABLED
            """
            return self.options['state']

        def set_logstate(self, logstate):
            """
            The logging state of the ACL.
            Default value: DISABLED
            """
            self.options['logstate'] = logstate

        def get_logstate(self):
            """
            The logging state of the ACL.
            Default value: DISABLED
            """
            return self.options['logstate']

        def set_ratelimit(self, ratelimit):
            """
            log message rate limit for acl rule.
            Default value: 100
            Minimum value =  1
            Maximum value =  10000
            """
            self.options['ratelimit'] = ratelimit

        def get_ratelimit(self):
            """
            log message rate limit for acl rule.
            Default value: 100
            Minimum value =  1
            Maximum value =  10000
            """
            return self.options['ratelimit']

        def get_hits(self):
            """
            The hits of this ACL.
            Default value: 0
            """
            return self.options['hits']

        def get_kernelstate(self):
            """
            The commit status of the ACL.
            Default value: 0
            """
            return self.options['kernelstate']

        # Operations methods

        @staticmethod
        def add(nitro, nsacl):
            """
            Use this API to add nsacl resources.
            """
            __nsacl = NSAcl()
            __nsacl.set_aclname(nsacl.get_aclname())
            __nsacl.set_aclaction(nsacl.get_aclaction())
            __nsacl.set_srcip(nsacl.get_srcip())
            __nsacl.set_srcipop(nsacl.get_srcipop())
            __nsacl.set_srcipval(nsacl.get_srcipval())
            __nsacl.set_srcport(nsacl.get_srcport())
            __nsacl.set_srcportop(nsacl.get_srcportop())
            __nsacl.set_srcportval(nsacl.get_srcportval())
            __nsacl.set_destip(nsacl.get_destip())
            __nsacl.set_destipop(nsacl.get_destipop())
            __nsacl.set_destipval(nsacl.get_destipval())
            __nsacl.set_destport(nsacl.get_destport())
            __nsacl.set_destportop(nsacl.get_destportop())
            __nsacl.set_destportval(nsacl.get_destportval())
            __nsacl.set_ttl(nsacl.get_ttl())
            __nsacl.set_srcmac(nsacl.get_srcmac())
            __nsacl.set_protocol(nsacl.get_protocol())
            __nsacl.set_protocolnumber(nsacl.get_protocolnumber())
            __nsacl.set_vlan(nsacl.get_vlan())
            __nsacl.set_Interface(nsacl.get_Interface())
            __nsacl.set_established(nsacl.get_established())
            __nsacl.set_icmptype(nsacl.get_icmptype())
            __nsacl.set_icmpcode(nsacl.get_icmpcode())
            __nsacl.set_priority(nsacl.get_priority())
            __nsacl.set_state(nsacl.get_state())
            __nsacl.set_logstate(nsacl.get_logstate())
            __nsacl.set_ratelimit(nsacl.get_ratelimit())
            return __nsacl.add_resource(nitro)

        @staticmethod
        def delete(nitro, nsacl):
            """
            Use this API to delete nsacl resources.
            """
            __nsacl = NSAcl()
            __nsacl.set_aclname(nsacl.get_aclname())
            nsresponse = __nsacl.delete_resource(nitro, nsacl.get_aclname())
            return nsresponse

        @staticmethod
        def update(nitro, nsacl):
            """
            Use this API to update nsacl resources.
            """
            __nsacl = NSAcl()
            __nsacl.set_aclname(nsacl.get_aclname())
            __nsacl.set_aclaction(nsacl.get_aclaction())
            __nsacl.set_srcip(nsacl.get_srcip())
            __nsacl.set_srcipop(nsacl.get_srcipop())
            __nsacl.set_srcipval(nsacl.get_srcipval())
            __nsacl.set_srcport(nsacl.get_srcport())
            __nsacl.set_srcportop(nsacl.get_srcportop())
            __nsacl.set_srcportval(nsacl.get_srcportval())
            __nsacl.set_destip(nsacl.get_destip())
            __nsacl.set_destipop(nsacl.get_destipop())
            __nsacl.set_destipval(nsacl.get_destipval())
            __nsacl.set_destport(nsacl.get_destport())
            __nsacl.set_destportop(nsacl.get_destportop())
            __nsacl.set_destportval(nsacl.get_destportval())
            __nsacl.set_srcmac(nsacl.get_srcmac())
            __nsacl.set_protocol(nsacl.get_protocol())
            __nsacl.set_protocolnumber(nsacl.get_protocolnumber())
            __nsacl.set_icmptype(nsacl.get_icmptype())
            __nsacl.set_icmpcode(nsacl.get_icmpcode())
            __nsacl.set_vlan(nsacl.get_vlan())
            __nsacl.set_Interface(nsacl.get_Interface())
            __nsacl.set_priority(nsacl.get_priority())
            __nsacl.set_state(nsacl.get_state())
            __nsacl.set_logstate(nsacl.get_logstate())
            __nsacl.set_ratelimit(nsacl.get_ratelimit())
            return __nsacl.update_resource(nitro)

        #@staticmethod
        #def unset(nitro, nsacl):
        #    """ generated source for method unset """
        #    unsetresource = nsacl()
        #    unsetresource.aclname = aclname
        #    return unsetresource.unset_resource(client, args)

        @staticmethod
        def enable(nitro, nsacl):
            """
            Use this API to enable a acl of given name.
            """
            __nsacl = NSAcl()
            __nsacl.set_aclname(nsacl.get_aclname())
            return __nsacl.perform_operation(nitro, "enable")

        @staticmethod
        def disable(nitro, nsacl):
            """
            Use this API to disable a acl of given name.
            """
            __nsacl = NSAcl()
            __nsacl.set_aclname(nsacl.get_aclname())
            return __nsacl.perform_operation(nitro, "disable")

        @staticmethod
        def get(nitro, nsacl):
            """
            Use this API to fetch nsacl resource of given name.
            """
            __nsacl = NSAcl()
            __nsacl.set_aclname(nsacl.get_aclname())
            __nsacl.get_resource(nitro, object_name=__nsacl.get_aclname())
            return __nsacl

        @staticmethod
        def get_all(nitro):
            """
            Use this API to fetch all configured nsacl resources.
            """
            __url = nitro.get_url() + NSAcl.get_resourcetype()
            __json_nsacls = nitro.get(__url).get_response_field(NSAcl.get_resourcetype())
            __nsacls = []
            for json_nsacl in __json_nsacls:
                    __nsacls.append(NSAcl(json_nsacl))
            return __nsacls



from nsbaseresource import NSBaseResource
__author__ = 'ndenev'


class NSPbrs(NSBaseResource):

        def __init__(self):

            super(NSPbrs, self).__init__()

            self.options = {}
            self.resourcetype = NSPbrs.get_resourcetype()

        @staticmethod
        def get_resourcetype():
            return "nspbrs"

        @staticmethod
        def renumber(nitro):
            """
            Use this API to renumber nspbrs.
            """
            __nsprbs = NSPbrs()
            return __nsprbs.perform_operation(nitro, "renumber")

        @staticmethod
        def clear(nitro):
            """
            Use this API to clear nspbrs.
            """
            __nspbrs = NSPbrs()
            return __nspbrs.perform_operation(nitro, "clear")

        @staticmethod
        def apply(nitro):
            """
            Use this API to apply nspbrs.
            """
            __nspbrs = NSPbrs()
            return __nspbrs.perform_operation(nitro, "apply")



from nsbaseresource import NSBaseResource

__author__ = 'ndenev@gmail.com'


class AuthTacacsAction(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(AuthTacacsAction, self).__init__()

                self.options = {'name': '',
                                'serverip': '',
                                'serverport': '',
                                'authtimeout': '',
                                'tacacssecret': '',
                                'authorization': '',
                                'accounting': '',
                                'auditfailedcmds': '',
                                'defaultauthenticationgroup': '',
                                # ReadOnly
                                'success': '',
                                'failure': ''}

                self.resourcetype = AuthTacacsAction.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options:
                                        self.options[key] = json_data[key]

        @staticmethod
        def get_resourcetype():
                return "authenticationtacacsaction"

        def set_name(self, name):
                """
                Name for the TACACS+ profile (action). Must begin with a letter, number, or the underscore character (_), and must contain only letters,
                numbers, and the hyphen (-), period (.) pound (#), space ( ), at (@), equals (=), colon (:), and underscore characters.
                Cannot be changed after TACACS profile is created.
                Minimum length = 1
                """
                self.options['name'] = name

        def get_name(self):
                return self.options['name']

        def set_serverip(self, serverip):
                """
                IP address assigned to the TACACS+ server.
                """
                self.options['serverip'] = serverip

        def get_serverip(self):
                return self.options['serverip']

        def set_serverport(self, serverport):
                """
                Port number on which the TACACS+ server listens for connections.
                Default value: 49
                """
                self.options['serverport'] = serverport

        def get_serverport(self):
                return self.options['serverport']

        def set_authtimeout(self, authtimeout):
                """
                Number of seconds the NetScaler appliance waits for a response from the TACACS+ server.
                Default value: 3
                """
                self.options['authtimeout'] = authtimeout

        def get_authtimeout(self):
                return self.options['authtimeout']

        def set_tacacssecret(self, tacacssecret):
                """
                Key shared between the TACACS+ server and the NetScaler appliance. Required for allowing the NetScaler appliance to communicate with the TACACS+ server.
                Minimum length = 1
                """
                self.options['tacacssecret'] = tacacssecret

        def get_tacacssecret(self):
                return self.options['tacacssecret']

        def set_authorization(self, authorization):
                """
                Use streaming authorization on the TACACS+ server.
                """
                self.options['authorization'] = authorization

        def get_authorization(self):
                return self.options['authorization']

        def set_accounting(self, accounting):
                """
                Whether the TACACS+ server is currently accepting accounting messages.
                """
                self.options['accounting'] = accounting

        def get_accounting(self):
                return self.options['accounting']

        def set_auditfailedcmds(self, auditfailedcmds):
                """
                The state of the TACACS+ server that will receive accounting messages.
                """
                self.options['auditfailedcmds'] = auditfailedcmds

        def get_auditfailedcmds(self):
                return self.options['auditfailedcmds']

        def set_defaultauthenticationgroup(self, defaultauthenticationgroup):
                """
                This is the default group that is chosen when the authentication succeeds in addition to extracted groups.
                """
                self.options['defaultauthenticationgroup'] = defaultauthenticationgroup

        def get_defaultauthenticationgroup(self):
                return self.options['defaultauthenticationgroup']

        def get_success(self):
                return self.options['success']

        def get_failure(self):
                return self.options['failure']

        # Operations methods
        @staticmethod
        def get(nitro, tacacsaction):
                """
                Use this API to fetch a configured authenticationtacacsaction resource by name.
                """
                __tacacsaction = AuthTacacsAction()
                __tacacsaction.set_name(tacacsaction.get_name())
                __tacacsaction.get_resource(nitro)
                return __tacacsaction

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all configured authenticationtacacsaction resources.
                """
                __url = nitro.get_url() + AuthTacacsAction.get_resourcetype()
                __json_tacacsactions = nitro.get(__url).get_response_field(AuthTacacsAction.get_resourcetype())
                __tacacsactions = []
                for json_tacacsaction in __json_tacacsactions:
                        __tacacsactions.append(AuthTacacsAction(json_tacacsaction))
                return __tacacsactions

        @staticmethod
        def add(nitro, tacacsaction):
                """
                Use this API to add authenticationtacacsaction.
                """
                __tacacsaction = AuthTacacsAction()
                __tacacsaction.set_name(tacacsaction.get_name())
                __tacacsaction.set_serverip(tacacsaction.get_serverip())
                __tacacsaction.set_serverport(tacacsaction.get_serverport())
                __tacacsaction.set_authtimeout(tacacsaction.get_authtimeout())
                __tacacsaction.set_tacacssecret(tacacsaction.get_tacacssecret())
                __tacacsaction.set_authorization(tacacsaction.get_authorization())
                __tacacsaction.set_accounting(tacacsaction.get_accounting())
                __tacacsaction.set_auditfailedcmds(tacacsaction.get_auditfailedcmds())
                __tacacsaction.set_defaultauthenticationgroup(tacacsaction.get_defaultauthenticationgroup())
                return __tacacsaction.add_resource(nitro)

        @staticmethod
        def delete(nitro, tacacsaction):
                """
                Use this API to delete authenticationtacacsaction of a given name.
                """
                __tacacsaction = AuthTacacsAction()
                __tacacsaction.set_name(tacacsaction.get_name())
                nsresponse = __tacacsaction.delete_resource(nitro)
                return nsresponse

        @staticmethod
        def update(nitro, tacacsaction):
                """
                Use this API to update a rewriteaction of a given name.
                """
                __tacacsaction = AuthTacacsAction()
                __tacacsaction.set_name(tacacsaction.get_name())
                __tacacsaction.set_serverip(tacacsaction.get_serverip())
                __tacacsaction.set_serverport(tacacsaction.get_serverport())
                __tacacsaction.set_authtimeout(tacacsaction.get_authtimeout())
                __tacacsaction.set_tacacssecret(tacacsaction.get_tacacssecret())
                __tacacsaction.set_authorization(tacacsaction.get_authorization())
                __tacacsaction.set_accounting(tacacsaction.get_accounting())
                __tacacsaction.set_auditfailedcmds(tacacsaction.get_auditfailedcmds())
                __tacacsaction.set_defaultauthenticationgroup(tacacsaction.get_defaultauthenticationgroup())
                return __tacacsaction.update_resource(nitro)

        # No unset functionality for now.



from nsbaseresource import NSBaseResource
__author__ = 'Aleksandar Topuzovic'


class NSServiceLBMonitorBinding(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSServiceLBMonitorBinding, self).__init__()
                self.options = {'policyname': '',
                                'name': '',
                                'monitor_name': '',
                                'monstatparam2': '',
                                'monstatcode': '',
                                'failedprobes': '',
                                'monstatparam3': '',
                                'totalprobes': '',
                                'responsetime': '',
                                'monstatparam1': '',
                                'monitor_state': '',
                                'monstate': '',
                                'totalfailedprobes': '',
                                'dup_weight': ''}

                self.resourcetype = NSServiceLBMonitorBinding.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

        @staticmethod
        def get_resourcetype():
                """
                Binding object showing the lbmonitor that can be bound to service.
                """
                return "service_lbmonitor_binding"

        # Read/write properties
        def set_policyname(self, policyname):
                """
                The DoS protection policy name must be bound to the service. Also, for DoS protection to work on a service, an appropriate policy must be bound to the service.
                Minimum length = 1
                """
                self.options['policyname'] = policyname

        def get_policyname(self):
                """
                The DoS protection policy name must be bound to the service. Also, for DoS protection to work on a service, an appropriate policy must be bound to the service.
                Minimum length = 1
                """
                return self.options['policyname']

        def set_name(self, name):
                """
                The name of the service to which the policy will be bound.
                Minimum length = 1
                """
                self.options['name'] = name

        def get_name(self):
                """
                The name of the service to which the policy will be bound.
                Minimum length = 1
                """
                return self.options['name']

        def set_monitor_name(self, monitorname):
                """
                The monitor Names.
                """
                self.options['monitor_name'] = monitorname

        def get_monitor_name(self):
                """
                The monitor Names.
                """
                return self.options['monitor_name']

        # Read only properties
        def get_monstatparam2(self):
                """
                Second parameter for use with message code.
                """
                return self.options['monstatparam2']

        def get_monstatcode(self):
                """
                The code indicating the monitor response.
                """
                return self.options['monstatcode']

        def get_failedprobes(self):
                """
                Number of the current failed monitoring probes.
                """
                return self.options['failedprobes']

        def get_monitorname(self):
                """
                Third parameter for use with message code.
                """
                return self.options['monstatparam3']

        def get_totalprobes(self):
                """
                The total number of probs sent.
                """
                return self.options['totalprobes']

        def get_responsetime(self):
                """
                Response time of this monitor.
                """
                return self.options['responsetime']

        def get_monstatparam1(self):
                """
                First parameter for use with message code.
                """
                return self.options['monstatparam1']

        def get_monitor_state(self):
                """
                The running state of the monitor on this service.
                """
                return self.options['monitor_state']

        def get_monstate(self):
                """
                The configured state (enable/disable) of the monitor on this server.
                """
                return self.options['monstate']

        def get_totalfailedprobes(self):
                """
                The total number of failed probs.
                """
                return self.options['totalfailedprobes']

        def get_dup_weight(self):
                """
                The weight of the monitor.
                """
                return self.options['dup_weight']

        @staticmethod
        def get(nitro, servicelbmonitorbinding):
                """
                Use this API to fetch service monitor binding resource of given name.
                """
                __url = nitro.get_url() + NSServiceLBMonitorBinding.get_resourcetype() + "/" + servicelbmonitorbinding.get_name()
                __json_servicelbmonitorbindings = nitro.get(__url).get_response_field(NSServiceLBMonitorBinding.get_resourcetype())
                __servicelbmonitorbindings = []
                for json_servicelbmonitorbinding in __json_servicelbmonitorbindings:
                        __servicelbmonitorbindings.append(NSServiceLBMonitorBinding(json_servicelbmonitorbinding))
                return __servicelbmonitorbindings



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'

class NSVLAN(NSBaseResource):

        # General Netscaler configuration object

        def __init__(self, json_data = None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSVLAN, self).__init__()

                self.options = {
                        'id' : '',
                        'ipv6dynamicrouting' : '',
                        'rnat' : '',
                        'portbitmap' : '',
                        'tagbitmap' : '',
                        'ifaces' : '',
                        'tagifaces' : '',
                }

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key]=json_data[key]

                self.resourcetype = NSVLAN.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "vlan"


        def set_id(self, id):
                self.options['id'] = id

        def get_id(self):
                return self.options['id']

        def set_ipv6dynamicrouting(self, ipv6dynamicrouting):
                self.options['ipv6dynamicrouting'] = ipv6dynamicrouting

        def get_ipv6dynamicrouting(self):
                return self.options['ipv6dynamicrouting']

        def get_rnat(self):
                return self.options['rnat']

        def get_portbitmap(self):
                return self.options['portbitmap']

        def get_tagbitmap(self):
                return self.options['tagbitmap']

        def get_ifaces(self):
                return self.options['ifaces']

        def get_tagifaces(self):
                return self.options['tagifaces']

        @staticmethod
        def add(nitro, vlan):
                __vlan = NSVLAN()
                __vlan.set_id(vlan.get_id())
                __vlan.set_ipv6dynamicrouting(vlan.get_ipv6dynamicrouting())
                return __vlan.add_resource(nitro)

        @staticmethod
        def update(nitro, vlan):
                __vlan = NSVLAN()
                __vlan.set_id(vlan.get_id())
                __vlan.set_ipv6dynamicrouting(vlan.get_ipv6dynamicrouting())
                return __vlan.update_resource(nitro)

        @staticmethod
        def delete(nitro, vlan):
                __vlan = NSVLAN()
                __vlan.set_id(vlan.get_id())
                nsresponse = __vlan.delete_resource(nitro, object_name=__vlan.get_id())
                return nsresponse

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all vlan resources that are configured on netscaler.
                """
                __url = nitro.get_url() + NSVLAN.get_resourcetype()
                __json_vlans = nitro.get(__url).get_response_field(NSVLAN.get_resourcetype())
                __vlans = []
                for json_vlan in __json_vlans:
                        __vlans.append(NSVLAN(json_vlan))
                return __vlans




__author__ = 'vllazarenko'

from nsbaseresource import NSBaseResource


class NSFeature(NSBaseResource):

# Configuration for NS Features.

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSFeature, self).__init__()
                self.options = {'feature': '',
                                'reqfeature': '',
                                'wl': '',
                                'sp': '',
                                'lb': '',
                                'cs': '',
                                'cr': '',
                                'sc': '',
                                'cmp': '',
                                'pq': '',
                                'ssl': '',
                                'gslb': '',
                                'hdosp': '',
                                'cf': '',
                                'ic': '',
                                'sslvpn': '',
                                'aaa': '',
                                'ospf': '',
                                'rip': '',
                                'bgp': '',
                                'rewrite': '',
                                'ipv6pt': '',
                                'appfw': '',
                                'responder': '',
                                'htmlinjection': '',
                                'push': ''}

                self.resourcetype = NSFeature.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

        @staticmethod
        def get_resourcetype():
                return "nsfeature"

        def set_feature(self, feature):
                self.options['feature'] = feature

        def get_feature(self):
                return self.options['feature']

        def get_reqfeature(self):
                return self.options['reqfeature']

        def get_wl(self):
                return self.options['wl']

        def get_sp(self):
                return self.options['sp']

        def get_lb(self):
                return self.options['lb']

        def get_cs(self):
                return self.options['cs']

        def get_cr(self):
                return self.options['cr']

        def get_sc(self):
                return self.options['sc']

        def get_cmp(self):
                return self.options['cmp']

        def get_pq(self):
                return self.options['pq']

        def get_ssl(self):
                return self.options['ssl']

        def get_gslb(self):
                return self.options['gslb']

        def get_hdosp(self):
                return self.options['hdosp']

        def get_cf(self):
                return self.options['cf']

        def get_ic(self):
                return self.options['ic']

        def get_sslvpn(self):
                return self.options['sslvpn']

        def get_aaa(self):
                return self.options['aaa']

        def get_ospf(self):
                return self.options['ospf']

        def get_rip(self):
                return self.options['rip']

        def get_bgp(self):
                return self.options['bgp']

        def get_rewrite(self):
                return self.options['rewrite']

        def get_ipv6pt(self):
                return self.options['ipv6pt']

        def get_appfw(self):
                return self.options['appfw']

        def get_responder(self):
                return self.options['responder']

        def get_htmlinjection(self):
                return self.options['htmlinjection']

        def get_push(self):
                return self.options['push']

        # Operations methods
        @staticmethod
        def disable(nitro, resource):
                """
                Use this API to disable server.
                """
                __resource = NSFeature()
                __resource.set_feature(resource.get_feature())
                return __resource.perform_operation(nitro, "disable")

        @staticmethod
        def enable(nitro, resource):
                """
                Use this API to enable server.
                """
                __resource = NSFeature()
                __resource.set_feature(resource.get_feature())
                return __resource.perform_operation(nitro, "enable")


from nsbaseresource import NSBaseResource

__author__ = 'ndenev'


class NSRoute(NSBaseResource):

        # General Netscaler configuration object

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """

                super(NSRoute, self).__init__()

                self.options = {'network': '',
                                'netmask': '',
                                'gateway': '',
                                'cost': '',
                                'td': '',
                                'distance': '',
                                'cost1': '',
                                'weight': '',
                                'advertise': '',
                                'protocol': '',
                                'msr': '',
                                'monitor': '',
                                'routetype': '',
                                'detail': '',
                                'gatewayname': '',
                                'type': '',
                                'dynamic': '',
                                'Static': '',
                                'permanent': '',
                                'direct': '',
                                'nat': '',
                                'lbroute': '',
                                'adv': '',
                                'tunnel': '',
                                'data': '',
                                'data0': '',
                                'flags': '',
                                'routeowners': '',
                                'retain': '',
                                'ospf': '',
                                'isis': '',
                                'rip': '',
                                'bgp': '',
                                'dhcp': '',
                                'advospf': '',
                                'advisis': '',
                                'advrip': '',
                                'advbgp': '',
                                'state': '',
                                'totalprobes': '',
                                'totalfailedprobes': '',
                                'failedprobes': '',
                                'monstatcode': '',
                                'monstatparam1': '',
                                'monstatparam2': '',
                                'monstatparam3': '',
                                '__count': ''}

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

                self.resourcetype = NSRoute.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "route"

        def set_network(self, network):
                self.options['network'] = network

        def get_network(self):
                return self.options['network']

        def set_netmask(self, netmask):
                self.options['netmask'] = netmask

        def get_netmask(self):
                return self.options['netmask']

        def set_gateway(self, gateway):
                self.options['gateway'] = gateway

        def get_gateway(self):
                return self.options['gateway']

        def set_cost(self, cost):
                self.options['cost'] = cost

        def get_cost(self):
                return self.options['cost']

        def set_td(self, td):
                self.options['td'] = td

        def get_td(self):
                return self.options['td']

        def set_distance(self, distance):
                self.options['distance'] = distance

        def get_distance(self):
                return self.options['distance']

        def set_cost1(self, cost1):
                self.options['cost1'] = cost1

        def get_cost1(self):
                return self.options['cost1']

        def set_weight(self, weight):
                self.options['weight'] = weight

        def get_weight(self):
                return self.options['weight']

        def set_advertise(self, advertise):
                self.options['advertise'] = advertise

        def get_advertise(self):
                return self.options['advertise']

        def set_protocol(self, protocol):
                self.options['protocol'] = protocol

        def get_protocol(self):
                return self.options['protocol']

        def set_msr(self, msr):
                self.options['msr'] = msr

        def get_msr(self):
            return self.options['msr']

        def set_monitor(self, monitor):
                self.options['monitor'] = monitor

        def get_monitor(self):
                return self.options['monitor']

        def set_routetype(self, routetype):
                self.options['routetype'] = routetype

        def get_routetype(self):
                return self.options['routetype']

        def set_detail(self, detail):
                self.options['detail'] = detail

        def get_detail(self):
                return self.options['detail']

        def get_gatewayname(self):
                return self.options['gatewayname']

        def get_type(self):
                return self.options['type']

        def get_dynamic(self):
                return self.options['dynamic']

        def get_static(self):
                return self.options['Static']

        def get_permanent(self):
                return self.options['permanent']

        def get_direct(self):
                return self.options['direct']

        def get_nat(self):
                return self.options['nat']

        def get_lbroute(self):
                return self.options['lbroute']

        def get_adv(self):
                return self.options['adv']

        def get_tunnel(self):
                return self.options['tunnel']

        def get_data(self):
                return self.options['data']

        def get_data0(self):
                return self.options['data0']

        def get_flags(self):
                return self.options['flags']

        def get_routeowners(self):
                return self.options['routeowners']

        def get_retain(self):
                return self.options['retain']

        def get_ospf(self):
                return self.options['ospf']

        def get_isis(self):
                return self.options['isis']

        def get_rip(self):
                return self.options['rip']

        def get_bgp(self):
                return self.options['bgp']

        def get_dhcp(self):
                return self.options['dhcp']

        def get_advospf(self):
                return self.options['advospf']

        def get_advisis(self):
                return self.options['advisis']

        def get_advrip(self):
                return self.options['advrip']

        def get_advbgp(self):
                return self.options['advbgp']

        def get_state(self):
                return self.options['state']

        def get_totalprobes(self):
                return self.options['totalprobes']

        def get_totalfailedprobes(self):
                return self.options['totalfailedprobes']

        def get_failedprobes(self):
                return self.options['failedprobes']

        def get_monstatcode(self):
                return self.options['monstatcode']

        def get_monstatparam1(self):
                return self.options['monstatparam1']

        def get_monstatparam2(self):
                return self.options['monstatparam2']

        def get_monstatparam3(self):
                return self.options['monstatparam3']

        @staticmethod
        def add(nitro, route):
                __route = NSRoute()
                __route.set_network(route.get_network())
                __route.set_netmask(route.get_netmask())
                __route.set_gateway(route.get_gateway())
                __route.set_cost(route.get_cost())
                __route.set_td(route.get_td())
                __route.set_distance(route.get_distance())
                __route.set_cost1(route.get_cost1())
                __route.set_weight(route.get_weight())
                __route.set_advertise(route.get_advertise())
                __route.set_protocol(route.get_protocol())
                __route.set_msr(route.get_msr())
                __route.set_monitor(route.get_monitor())
                return __route.add_resource(nitro)

        @staticmethod
        def clear(nitro, route):
                __route = NSRoute()
                __route.set_routetype(route.get_routetype())
                return __route.perform_operation(nitro, "clear")

        @staticmethod
        def delete(nitro, route):
                __route = NSRoute()
                __route.set_netmask(route.get_netmask())
                __route.set_gateway(route.get_gateway())
                __route.set_td(route.get_td())
                nsresponse = __route.delete_resource(nitro, object_name=route.get_network())
                return nsresponse

        @staticmethod
        def update(nitro, route):
                __route = NSRoute()
                __route.set_network(route.get_network())
                __route.set_netmask(route.get_netmask())
                __route.set_gateway(route.get_gateway())
                __route.set_td(route.get_td())
                __route.set_distance(route.get_distance())
                __route.set_cost1(route.get_cost1())
                __route.set_weight(route.get_weight())
                __route.set_advertise(route.get_advertise())
                __route.set_protocol(route.get_protocol())
                __route.set_msr(route.get_msr())
                __route.set_monitor(route.get_monitor())
                return __route.update_resource(nitro)

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all nsroute resources that are configured on netscaler.
                """
                __url = nitro.get_url() + NSRoute.get_resourcetype()
                __json_routes = nitro.get(__url).get_response_field(NSRoute.get_resourcetype())
                __routes = []
                for json_route in __json_routes:
                        __routes.append(NSRoute(json_route))
                return __routes

        # No route count functionality for now.
        # No unset functionality for now.



from nsbaseresource import NSBaseResource
__author__ = 'vlazarenko'


class NSLBMonitorServiceBinding(NSBaseResource):

        # Binding monitors to lb vservers

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSLBMonitorServiceBinding, self).__init__()
                self.options = {'monitorname': '',
                                'servicename': '',
                                'dup_state': '',
                                'dup_weight': '',
                                'servicegroupname': '',
                                'state': '',
                                'weight': ''}

                self.resourcetype = NSLBMonitorServiceBinding.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options.keys():
                                        self.options[key] = json_data[key]

        @staticmethod
        def get_resourcetype():
                return "lbmonitor_service_binding"

        def set_monitorname(self, monitorname):
                self.options['monitorname'] = monitorname

        def get_monitorname(self):
                return self.options['monitorname']

        def set_servicename(self, servicename):
                self.options['servicename'] = servicename

        def get_servicename(self):
                return self.options['servicename']

        def set_dup_state(self, dup_state):
                self.options['dup_state'] = dup_state

        def get_dup_state(self):
                return self.options['dup_state']

        def set_dup_weight(self, dup_weight):
                self.options['dup_weight'] = dup_weight

        def get_dup_weight(self):
                return self.options['dup_weight']

        def set_servicegroupname(self, servicegroupname):
                self.options['servicegroupname'] = servicegroupname

        def get_servicegroupname(self):
                return self.options['servicegroupname']

        def set_state(self, state):
                self.options['state'] = state

        def get_state(self):
                return self.options['state']

        def set_weight(self, weight):
                self.options['weight'] = weight

        def get_weight(self):
                return self.options['weight']

        @staticmethod
        def add(nitro, resource):
                """
                Use this API to add lbmonitor_service_binding.
                """
                __resource = NSLBMonitorServiceBinding()
                __resource.set_monitorname(resource.get_monitorname())
                __resource.set_servicename(resource.get_servicename())
                __resource.set_dup_state(resource.get_dup_state())
                __resource.set_dup_weight(resource.get_dup_weight())
                __resource.set_servicegroupname(resource.get_servicegroupname())
                __resource.set_state(resource.get_state())
                __resource.set_weight(resource.get_weight())
                return __resource.update_resource(nitro)
        
        @staticmethod
        def delete(nitro, resource):
                """
                Use this API to delete lbmonitor_service_binding of a given name.
                """
                __resource = NSLBMonitorServiceBinding()
                __resource.set_monitorname(resource.get_monitorname())
                __resource.set_servicename(resource.get_servicename())
                __resource.set_servicegroupname(resource.get_servicegroupname())
                nsresponse = __resource.delete_resource(nitro, object_name=__resource.get_monitorname())
                return nsresponse



from nsbaseresource import NSBaseResource

__author__ = 'ndenev@gmail.com'


class AuthTacacsPolicy(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(AuthTacacsPolicy, self).__init__()

                self.options = {'name': '',
                                'rule': '',
                                'reqaction': ''}

                self.resourcetype = AuthTacacsPolicy.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if key in self.options:
                                        self.options[key] = json_data[key]

        @staticmethod
        def get_resourcetype():
                return "authenticationtacacspolicy"

        def set_name(self, name):
                """
                Name for the TACACS+ policy. Must begin with a letter, number, or the underscore character (_), and must contain only letters,
                numbers, and the hyphen (-), period (.) pound (#), space ( ), at (@), equals (=), colon (:), and underscore characters.
                Cannot be changed after TACACS+ policy is created.
                Minimum length = 1
                """
                self.options['name'] = name

        def get_name(self):
                return self.options['name']

        def set_rule(self, rule):
                """
                Name of the NetScaler named rule, or a default syntax expression, that the policy uses to determine whether to attempt to authenticate the user with the TACACS+ server.
                Minimum length = 1
                """
                self.options['rule'] = rule

        def get_rule(self):
                return self.options['rule']

        def set_reqaction(self, reqaction):
                """
                Name of the TACACS+ action to perform if the policy matches.
                Minimum length = 1
                """
                self.options['reqaction'] = reqaction

        def get_reqaction(self):
                return self.options['reqaction']

        # Operations methods
        @staticmethod
        def get(nitro, tacacspolicy):
                """
                Use this API to fetch a configured authenticationtacacspolicy resource by name.
                """
                __tacacspolicy = AuthTacacsPolicy()
                __tacacspolicy.set_name(tacacspolicy.get_name())
                __tacacspolicy.get_resource(nitro)
                return __tacacspolicy

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all configured authenticationtacacspolicy resources.
                """
                __url = nitro.get_url() + AuthTacacsPolicy.get_resourcetype()
                __json_tacacspolicies = nitro.get(__url).get_response_field(AuthTacacsPolicy.get_resourcetype())
                __tacacspolicies = []
                for json_tacacspolicy in __json_tacacspolicies:
                        __tacacspolicies.append(AuthTacacsPolicy(json_tacacspolicy))
                return __tacacspolicies

        @staticmethod
        def add(nitro, tacacspolicy):
                """
                Use this API to add authenticationtacacspolicy.
                """
                __tacacspolicy = AuthTacacsPolicy()
                __tacacspolicy.set_name(tacacspolicy.get_name())
                __tacacspolicy.set_rule(tacacspolicy.get_rule())
                __tacacspolicy.set_reqaction(tacacspolicy.get_reqaction())
                return __tacacspolicy.add_resource(nitro)

        @staticmethod
        def delete(nitro, tacacspolicy):
                """
                Use this API to delete authenticationtacacspolicy of a given name.
                """
                __tacacspolicy = AuthTacacsPolicy()
                __tacacspolicy.set_name(tacacspolicy.get_name())
                nsresponse = __tacacspolicy.delete_resource(nitro)
                return nsresponse

        @staticmethod
        def update(nitro, tacacspolicy):
                """
                Use this API to update a rewritepolicy of a given name.
                """
                __tacacspolicy = AuthTacacsPolicy()
                __tacacspolicy.set_name(tacacspolicy.get_name())
                __tacacspolicy.set_rule(tacacspolicy.get_rule())
                __tacacspolicy.set_reqaction(tacacspolicy.get_reqaction())
                return __tacacspolicy.update_resource(nitro)

        # No unset functionality for now.
